#!/usr/bin/env python

import subprocess, os

def runBin(part1,part2,args=None,options=None):
  if options is None:
    options,args = parseArgs(args)
  
  path = os.getenv('NAO_HOME') + '/build2/build/build-%s%s/%s/sdk/bin/%s'
  arch = 'atom' if options.atom else 'geode'
  release = '' if options.debug else '-release'

  cmd = os.path.normpath(path % (arch,release,part1,part2))
  runCommand(cmd,options,args)

def runCommand(cmd,options=None,args=None,ldAdditional=''):
  if options is None:
    options,args = parseArgs(args)

  fullCmd = [cmd] + args
  if options.gdb:
    fullCmd = ['gdb','-ex','run','--args'] + fullCmd

  arch = 'atom' if options.atom else 'geode'
  ldpath = os.getenv('NAO_HOME') + ('/naoqi/crosstoolchain/%s/sysroot/usr/lib/' % arch) + ldAdditional

  os.putenv('LD_LIBRARY_PATH',os.getenv('LD_LIBRARY_PATH','') + ':' + ldpath)
  p = subprocess.Popen(fullCmd)
  try:
    p.wait()
  finally:
    if p.poll() == None:
      p.terminate()

def parseArgs(args=None):
  from optparse import OptionParser
  import sys
  usage = 'usage: %prog [options] [teamNum [playerNum]]'
  parser = OptionParser(usage)
  parser.add_option('--geode',action='store_false',dest='atom',help='run the geode binaries',default=True)
  parser.add_option('--atom',action='store_true',dest='atom',help='run the atom binaries',default=True)
  parser.add_option('--gdb',action='store_true',dest='gdb',help='run gdb on the binaries (you may also want to use --debug to use the debug binaries',default=False)
  parser.add_option('--debug',action='store_true',dest='debug',help='use the debug binaries',default=False)
  if args is None:
    args = sys.argv[1:]
    otherArgs = []
    i = 0
    while i < len(args):
      if (args[i][0] == '-') and (args[i][1] != '-'):
        otherArgs.append(args[i])
        del args[i]
      else:
        i += 1
  options,args = parser.parse_args(args)
  args = otherArgs + args
  return options,args

