/**
* \file Platform/linux/NaoCamera.h
* Interface to the Nao camera using linux-uvc.
* \author Colin Graf
*/

#ifndef _NaoCamera_H_
#define _NaoCamera_H_

#include "V4L2Constants.h"
#include <common/RobotInfo.h>
#include <constants/ImageConstants.h>
#include <cstring>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <iostream>

// struct buffer {
//   void * start;
//   size_t length;
// };

/**
* \class NaoCamera
* Interface class to the Nao camera.
*/
class NaoCamera
{
public:
  virtual void updateBuffer();
  virtual unsigned char* getImage() const;
  virtual unsigned getTimeStamp() const;
  virtual int getControlSetting(unsigned int id);
  virtual bool setControlSetting(unsigned int id, int value);
  virtual void setCameraParams();
  virtual void getCameraParams();
  virtual void reset();
  virtual bool selfTest();
  virtual ~NaoCamera();

protected:
    bool vflip_, hflip_;
    std::string device_path_;
    CameraParams& camera_params_;
    CameraParams& read_camera_params_;
    NaoCamera(CameraParams&, CameraParams&);
    void init();

private:
  int videoDeviceFd;
  bool initialized;
  enum
  {
    frameBufferCount = 3, /**< Amount of available frame buffers. */
  };
  void* mem[frameBufferCount]; /**< Frame buffer addresses. */
  int memLength[frameBufferCount]; /**< The length of each frame buffer. */
  struct v4l2_buffer* buf; /**< Reusable parameter struct for some ioctl calls. */
  struct v4l2_buffer* currentBuf; /**< The last dequeued frame buffer. */
  unsigned timeStamp, /**< Timestamp of the last captured image. */
           storedTimeStamp; /**< Timestamp when the next image recording starts. */

  // void openDevice();
  // void initDevice();
  // void closeDevice();
  // void initMMAP();
  // void uninitDevice();
  // void startCapture();
  // void stopCapture();
  // void readFrame();
  // int xioctl(int fd, int request, void * arg);
  // void errnoExit(const char* s);

  // buffer* buffers;
  // unsigned int n_buffers;

  void setDefaultSettings();
  void initOpenVideoDevice();
  void initSetImageFormat();
  void initSetFrameRate();
  void initRequestAndMapBuffers();
  void initQueueAllBuffers();
  void unmapAndFreeBuffers();
  void enableStreaming();
  void disableStreaming();

  void displayAvailableSettings();
  void enumerateSettingMenu();

  v4l2_queryctrl queryctrl;
  v4l2_querymenu querymenu;

};

#endif // _NaoCamera_H_
