#ifndef _UKF_h_DEFINED
#define _UKF_h_DEFINED

#include <math.h>
#include <common/NMatrix.h>
#include <common/RobotInfo.h>
#include <math/Geometry.h>

class TextLogger;

template <class T>
inline T NORMALISE(T theta){
    return atan2(sin(theta), cos(theta));
}

template <class T>
inline T CROP(T num, T low, T high){
  if (num < low) num = low;
  else if (num > high) num = high;
  return num;
}

enum ukfUpdateResult {
  UKF7_OUTLIER,
  UKF7_OK,
  UKF7_OFF  // return because this is turned off by params
};

using namespace std;

struct SmallUKFParams {
  float R_ball_theta;
  float R_ball_range_offset;
  float R_ball_range_relative;

  float kappa;
  float ball_decay_rate;
  float outlier_rejection_thresh;
  float kick_outlier_rejection_thresh;
  float seen_ball_outlier_rejection_thresh;
  float shared_ball_outlier_rejection_thresh;
  float keeper_outlier_rejection_thresh_diff;

  // process noise
  float robot_pos_noise;
  float robot_orient_noise;
  float ball_pos_noise;
  float ball_vel_noise;
  float robot_bump_sd_factor;
  float fallen_robot_sd_factor;
  float fallen_ball_sd_factor;
  float standing_robot_sd_factor;
  float moving_ball_sd_factor;

  // std's used when resetting things
  float init_sd_x;
  float init_sd_y;
  float init_sd_theta;
  float init_sd_vel;

  float reset_sd_pos;
  float reset_sd_theta;
  float reset_sd_ball;

};


class UKF {
 public:
  // Constructor
  UKF();

  // Functions

  // Update functions
  void timeUpdate(float odometeryForward, float odometeryLeft, float odometeryTurn, float timePassed, bool bump, bool fallen, bool standing);
  ukfUpdateResult ballmeas(float Ballmeas, float theta_Ballmeas);
  ukfUpdateResult ballmeasWithBearing(float Ballmeas, float theta_Ballmeas);
  ukfUpdateResult fieldObjectmeas(float distance, float bearing,float objX,float objY, float distanceErrorOffset, float distanceErrorRelative, float bearingError);
  ukfUpdateResult ballKickUpdate(float velocity, float relBearing, float velError, float bearingError);
  ukfUpdateResult fieldObjectMeasWithBearing(float distance, float bearing,float objX,float objY, float distanceErrorOffset, float distanceErrorRelative, float bearingError);
  ukfUpdateResult lineObjectMeas(float distance, float bearing,float objX, float objY, float lineOrient, float distanceErrorOffset, float distanceErrorRelative, float bearingError);

  ukfUpdateResult linear2MeasurementUpdate(float Y1,float Y2, float SR11, float SR12, float SR22, int index1, int index2, int mate, bool weAreKeeper);

  // Data retrieval
  float sd(int Xi);
  float variance(int Xi);
  float getState(int stateID);
  NMatrix GetBallSR();
  float getDistanceToPosition(float posX, float posY);
  float getBearingToPosition(float posX, float posY);

  // Utility
  void init();
  void reInit();
  void setParams(SmallUKFParams u);
  void Reset();
  bool clipState(int stateIndex, float minValue, float maxValue);
  void setTextLogger(TextLogger* tf);
  

  // Variables

  // Multiple Models - Model state Description.
  double alpha;
  bool isActive;
  bool toBeActivated;

  int framesSinceAmbig;
  int ballFramesSinceAmbig;

  // State data
  NMatrix updateUncertainties;     // Update Uncertainty.          (A matrix)
  NMatrix stateEstimates;          // State estimates.             (Xhat NMatrix)
  NMatrix stateStandardDeviations; // Standard Deviation NMatrix.   (S NMatrix)

  // Constants
  int nStates;                    // Number of states.            (Constant)
  NMatrix sqrtOfTestWeightings;    // Square root of W             (Constant)
  NMatrix sqrtOfProcessNoise;      // Square root of Process Noise (Q matrix). (Constant)
  NMatrix sqrtOfProcessNoiseReset; // Square root of Q for reset.  (Conastant)

  // Tuning Values 
  SmallUKFParams ukfParams;

  TextLogger* textlogger;

};

#endif
