require 'commands'
require 'task'
require 'poses'
require 'cfgstiff'
module(..., package.seeall)


function adjustKickAlignment(inAlignment,id,kickChoice)
  local alignment = deepcopy(inAlignment)
  -- nothing robot specific yet
  return alignment
end

CompTask('body_AlignToBallWithTargetWalk',{
           init = true,
           desiredX = 155,--165, -- overwritten
           alignment = kicks.kickData[core.FwdShortStraightKick].alignment,
           --yOffset = 50,
           desiredY = 50, -- overwritten
           done = false,
           doneCount = 0,
           heading = 0,
           currentFoot = core.Kick_RIGHT,
           desiredFoot = core.Kick_RIGHT,
           needToSend = true,
           xError = 500,
           lastSendBallLoc = core.Point2D(0,0),
           kickChoice = core.FwdShortStraightKick,
         })
function body_AlignToBallWithTargetWalk:choose(as)
  self.alignment = adjustKickAlignment(self.alignment,robot_state.robot_id_,self.kickChoice)
  self.desiredX = self.alignment.des_x

  -- set current foot to match the one passed in from the kick
  if (self.init) then
    UTdebug.log(10, "target approach init")
    self.currentFoot = self.desiredFoot
    if (self.currentFoot == core.Kick_RIGHT) then
      self.desiredY = -self.alignment.des_y
    else
      self.desiredY = self.alignment.des_y
    end
    self.init = false
    self.needToSend = true
    self.doneCount = 0
  end

  local ball = world_objects:getObjPtr(core.WO_BALL)
  local dx = localizationC.filtered_close_ball_.x
  local dy = localizationC.filtered_close_ball_.y

  local realDesiredX = self.desiredX

  local origXError = (dx - self.desiredX)
  local xError = (dx - realDesiredX)

  if (self.currentFoot == core.Kick_RIGHT) then
    self.desiredY = -self.alignment.des_y
  else
    self.desiredY = self.alignment.des_y
  end

  local yError = (dy - self.desiredY)
  
  local yAlignError = yError
  if (self.currentFoot == core.Kick_RIGHT) then
    yAlignError = -yError
  end

  -- calculate body target from ball
  UTdebug.log(10, "Ball",dx,dy, " desired",realDesiredX, self.desiredY, "error",xError,yError,yAlignError)

  behavior_mem.targetPt = core.Point2D(xError,yError)
  behavior_mem.useTarget = true
  local me = world_objects:getObjPtr(robot_state.WO_SELF)
  behavior_mem.absTargetPt = behavior_mem.targetPt:relativeToGlobal(me.loc, me.orientation)
  behavior_mem.useAbsTarget = true
  behavior_mem.useTargetBearing = false

  local doneThreshold = 3

  local extra = 0
  if (game_state.isPenaltyKick and game_state.secsRemaining > 10) then
    extra = 10
  end


  -- check if we're close to target for some # of frames
  -- normal
  local sideBuffer = 0 --12
  local fwdBuffer = 0 --7
  local incrementDoneCounter = ball.seen and origXError < (self.alignment.max_stop_x_err + fwdBuffer) and yAlignError > (self.alignment.min_stop_y_err - sideBuffer) and yAlignError < (self.alignment.max_stop_y_err + sideBuffer)
  
  if incrementDoneCounter then
    if (self.doneCount < doneThreshold + 2) then
      self.doneCount = self.doneCount + 1
    end
  elseif (self.doneCount > 0) then
    self.doneCount = self.doneCount - 1
  end

  self.done = (self.doneCount >= doneThreshold)
  UTdebug.log(10, "done check", ball.seen, origXError, yAlignError, self.alignment.max_stop_x_err, self.alignment.min_stop_y_err, self.alignment.max_stop_y_err, self.doneCount, doneThreshold)

  -- also done if we're standing now
   UTdebug.log(10,"MTBOF", xError, yError, origXError, realDesiredX, ball.distance, self.currentFoot, self.needToSend, self.doneCount, self.done)
  
  -- so approach can know about our xError
  self.xError = xError

  -- dont change target when not seeing ball
  if (not ball.seen) then
    self.done = false
    return head_LookForBall:set()
  end
  
  -- check error 
  local ballMovement = core.Point2D(dx,dy):getDistanceTo(self.lastSendBallLoc)

  -- if ball has moved more than 6 cm... send command
  if (self.needToSend or xError > 85 or ballMovement > 60) then
    UTdebug.log(10, "need to send, xError, ballMove", self.needToSend, xError, ballMovement)
    self.doneCount = 0
    self.done = false
  end
  
  local pedantic = false
  if (math.abs(xError) < 40) and (yError < 20) then
    -- turn on pedantic at the end
    pedantic = true
  end
  walk_request:setWalkTarget(xError,yError,0,pedantic)
  self.needToSend = false
  self.lastSendBallLoc = core.Point2D(dx,dy)

  return head_LookForBall:set()

end

function body_AlignToBallWithTargetWalk:finished()
  return self.done
end



-------------------------------
-- set body pose and stiffness


PrimTask('body_ToPose',{
           pose = sittingPoseV3,
           init = true,
           time = 3.0,
           reverseSides = false
         })
function body_ToPose:run()
  -- only have to send command first time
  if (self.init) then
    self:resetTime()
    self.init = false

    local val

    -- for each body joint (not head)
    for i=2, core.NUM_JOINTS-1 do
      -- do interp to given angle
      val = DEG_T_RAD*getPoseJoint(i,self.pose,self.reverseSides)
      joint_commands:setJointCommand(i, val)
    end

    -- set time and send_angles_
    joint_commands.send_body_angles_ = true
    joint_commands.body_angle_time_ = self.time * 1000.0

    -- turn off walk request and kick request
    walk_request:noWalk()
    kick_request:setNoKick()

  end
end

function body_ToPose:finished()
  return (self:getTime() > self.time+0.1) --0.05)
end

PrimTask('body_ToPoseSequence',{
    init = true,
    poses = {sittingPoseV3},
    times = {1.0},
    stiffness = cfgStiffOne,
    state = 0
  })
function body_ToPoseSequence:run()
  if self.init then
    self.init = false
    self.state = 0
    commands.setStiffnessCommands(self.stiffness,0.1)
  end
  walk_request:noWalk()
  kick_request:setNoKick()

  if self:finished() then
    return
  end

  if (self.state == 0) or self:getTime() > self.times[self.state] then
    self:resetTime()
    self.state = self.state + 1
    if self:finished() then
      return 
    end
  end

  local val
  -- for each body joint (not head)
  for i=2, core.NUM_JOINTS-1 do
    -- do interp to given angle
    val = DEG_T_RAD*self.poses[self.state][i]
    joint_commands:setJointCommand(i, val)
  end
  joint_commands.send_body_angles_ = true
  local time = (self.times[self.state] - self:getTime()) * 1000
  if self.poses[self.state] == self.poses[self.state-1] then
    time = 0
  end
  joint_commands.body_angle_time_ = time

  return as
end

function body_ToPoseSequence:finished()
  return self.state > table.getn(self.poses)
end

-- A helper func for reversing the sides of a pose
function getPoseJoint(joint,pose,reversed)
  if not(reversed) then
    return pose[joint]
  end

  -- because there's only one groin joint, just handle it separately, and don't reverse it
  if (joint == core.LHipYawPitch) or (joint == core.RHipYawPitch) then
    return pose[joint]
  end

  local val
  -- get the joint value for the joint on the opposite side
  if joint >= core.LShoulderPitch and joint <= core.LElbowRoll then
    --left arm
    val = pose[joint+(core.RShoulderPitch-core.LShoulderPitch)]
  elseif joint >= core.LHipYawPitch and joint <= core.LAnkleRoll then
    --left leg
    val = pose[joint+(core.RHipYawPitch-core.LHipYawPitch)]
  elseif joint >= core.RHipYawPitch and joint <= core.RAnkleRoll then
    --right leg
    val = pose[joint-(core.RHipYawPitch-core.LHipYawPitch)]
  elseif joint >= core.RShoulderPitch and joint <= core.RElbowRoll then
    --right arm
    val = pose[joint-(core.RShoulderPitch-core.LShoulderPitch)]
  end

  -- reverse the roll directions
  local directionReversedJoints = {core.LHipRoll,core.RHipRoll,core.LAnkleRoll,core.RAnkleRoll,core.LShoulderRoll,core.RShoulderRoll,core.LElbowRoll,core.RElbowRoll,core.LElbowYaw,core.RElbowYaw}
  for i = 1,#directionReversedJoints do
    if joint == directionReversedJoints[i] then
      return -val
    end
  end
  return val
end

PrimTask('body_PrintPose',{
    init = true
  })
function body_PrintPose:run()
  UTdebug.log(0,'pose = {}')
  for i = 0,core.NUM_JOINTS-1 do
    UTdebug.log(0,'pose[core.'.. core.getJointName(i) .. '] = ',RAD_T_DEG * joint_angles[i])
  end
end

function body_PrintPose:finished()
  return true
end

CompTask('body_MoveHome',{
           init = true,
           done = false
         })
function body_MoveHome:choose(as)
  keeper = core.Pose2D(0.0, -2800.0 ,0.0)  -- Orientation, X,Y

  -- Rules: keeper must be inside penalty area, no other players can be
  -- On our kick off, only one player inside circle, others anywhere in half
  -- On opp kick off, players behind cross (-1200)

  -- for our kickoff
  kickoff_attacker         = core.Pose2D(0,  -300,0)
  kickoff_left_wing        = core.Pose2D( 0, -300, 1500)
  kickoff_right_wing       = core.Pose2D( 0, -300,-1500)
  kickoff_left_defender    = core.Pose2D( 0,-1600, 600)
  kickoff_right_defender   = core.Pose2D( 0,-1600,-600)

  receive_middle = core.Pose2D(0,-1650,0)
  receive_left   = core.Pose2D(0,-1700,900)
  receive_right  = core.Pose2D(0,-1700,-500)


  if (as == body_MoveToPoint) then
    if (as:finished()) then
      self.done = true
    end
  end


  local desired_pos = core.Pose2D()

  if (robot_state.role_ == core.KEEPER) then
    desired_pos = keeper
  elseif (robot_state.role_ == core.CHASER) then
    if (game_state.ourKickOff) then
      desired_pos = kickoff_attacker
  
    else
      desired_pos = receive_middle
    end
  elseif (robot_state.role_ == core.SUPPORTER) then
    if (game_state.ourKickOff) then
      desired_pos = kickoff_left_wing
    else
      desired_pos = receive_left
    end
  else -- role 3
    if (game_state.ourKickOff) then
      desired_pos = kickoff_right_defender
    else
      desired_pos = receive_right
    end
  end

  return body_MoveToPoint:set{maxVel = 1.0,des = core.Point2D(desired_pos.translation.x,desired_pos.translation.y), desTheta = DEG_T_RAD*desired_pos.rotation}

end

function body_MoveHome:finished()
  return self.done
end


PrimTask('body_MoveToPoint', {
           des = core.Point2D(0,0),
           desTheta = 0.0,
           lookAtPoint = false,
           look = core.Point2D(0,0),
           close = false,
           walkStraight = true, -- if true, turns towards point and goes, otherwise tries to maintain target orientation
           maxVel = 1,
           maxTurn = 0.5,
           maxBackward = 0.25,
           orientThresh = 600.0,
           lastTurn = 0,
           doneCount = 0,
           doneDist = 50,
           doneAngle = 5.0,
           doneThresh = 20,
           --how close do we have to get to start slowing from max vel
           distThresh = 600,
           turnThresh = 1.8, -- 103 degrees
           turnToFace = false,
           lastSprint = false,
           maxSprintTurn = 0.2
         })
function body_MoveToPoint:run()

  -- get our location
  local selfRobot = world_objects:getObjPtr(robot_state.WO_SELF)
  local loc = selfRobot.loc
  local desTheta = self.desTheta
  self.turnToFace = false

  -- sprint in past orient thresh, so that we dont trash between modes
  local orientThresh = self.orientThresh
  if (self.lastSprint) then
    orientThresh = orientThresh - 200
  end

  -- two versions, one that looks at a point and one that does not
  if (self.lookAtPoint) then
    -- calculate bearing to point
    desTheta = loc:getBearingTo(self.look, 0)

  end

  -- even look at point may walk straight if far enough away
  if (self.walkStraight and loc:getDistanceTo(self.des) > orientThresh) then
    -- get the bearing to walk straight to the point
    desTheta = loc:getBearingTo(self.des, 0)
    self.turnToFace = true

    UTdebug.log(20, "Turn straight toward target point", desTheta*RAD_T_DEG)

  end

  self.close = (loc:getDistanceTo(self.des) < self.doneDist) and (math.abs(selfRobot.orientation - desTheta) < self.doneAngle*DEG_T_RAD)

  if (self.close) then
    self.doneCount = self.doneCount + 1
  elseif (self.doneCount > 0) then
    self.doneCount = self.doneCount - 1
  end

  -- get error in these
  local diff = self.des - loc

  local diffTheta = core.normalizeAngle(desTheta - selfRobot.orientation)

  -- bias towards last turn direction if its all the way around
  if (math.abs(diffTheta) > DEG_T_RAD * 160.0) then
    if (self.lastTurn > 0 and diffTheta < 0) then
      --UTdebug.log(20, "Over 160 deg, stay with last turn", diffTheta, self.lastTurn)
      diffTheta = core.normalizeAngle(diffTheta + 30.0*DEG_T_RAD)
    elseif (self.lastTurn < 0 and diffTheta > 0) then
      --UTdebug.log(20, "Over 160 deg, stay with last turn", diffTheta, self.lastTurn)
      diffTheta = core.normalizeAngle(diffTheta - 30.0*DEG_T_RAD)
    end
  end

  -- rotate to put into robot's coordinate frame
  diff:rotate(-selfRobot.orientation)

  behavior_mem.targetPt = diff
  behavior_mem.useTarget = true
  behavior_mem.absTargetPt = self.des
  behavior_mem.useAbsTarget = true
  behavior_mem.useTargetBearing = false

  UTdebug.log(20, "self, des, diff, diffTheta", loc, RAD_T_DEG*selfRobot.orientation,self.des, diff, RAD_T_DEG*diffTheta, self.close)

  -- set velocity
  local velX, velY, velTheta
  if (diff.x > self.distThresh) then
    velX = self.maxVel
  elseif (diff.x < -self.distThresh) then
    velX = -self.maxBackward
  else
    velX = self.maxVel * diff.x / self.distThresh
  end

  if (diff.y > self.distThresh) then
    velY = self.maxVel
  elseif (diff.y < -self.distThresh) then
    velY = -self.maxVel
  else
    velY = self.maxVel * diff.y / self.distThresh
  end

  if (diffTheta > self.turnThresh) then
    velTheta = self.maxTurn
  elseif (diffTheta < -self.turnThresh) then
    velTheta = -self.maxTurn
  else
    velTheta = self.maxTurn * diffTheta / self.turnThresh
  end

  self.lastTurn = 1
  if (velTheta < 0) then
    self.lastTurn = -1
  end

  local localMaxSprintTurn = self.maxSprintTurn + 0.1
  -- go into sprint more if we're really far
  if (loc:getDistanceTo(self.des) > 2500) then
    localMaxSprintTurn = localMaxSprintTurn + 0.1
  end

  -- if we're roughly facing point, lets sprint
  if (self.walkStraight and self.turnToFace) then
    UTdebug.log (10, "in walk straight, turn error is ", diffTheta *RAD_T_DEG, self.lastSprint, self.lastTurn, velTheta)
    -- if we were turning, continue turn vel under 0.14
    -- if we were sprinting, start turning if turn vel over 0.2
    if ((not self.lastSprint and math.abs(velTheta) > (0.7*localMaxSprintTurn)) or (math.abs(velTheta) > localMaxSprintTurn)) then
      velX = 0
      velY = 0
      UTdebug.log(10, "turn to face point first", diffTheta * RAD_T_DEG, "new vel", velX, velY, velTheta)
      self.lastSprint = false
      -- otherwise, sprint
    else
      velY = 0
      -- still crop to actual limit
      velTheta = core.crop(velTheta, -self.maxSprintTurn, self.maxSprintTurn)
      UTdebug.log(10, "sprint to point", diffTheta * RAD_T_DEG, "new vel", velX, velY, velTheta)
      self.lastSprint = true
    end
  else
    self.lastSprint = false
  end

  commands.setWalkVelocity(velX, velY, velTheta, self.lastSprint)
end

function body_MoveToPoint:finished()
  return (self.doneCount > self.doneThresh)
end
