require 'task'
module(..., package.seeall);


PrimTask('head_MoveHead',{
           head_init = true,
           time = 2.0,
           pan = 0.0
         })
function head_MoveHead:run()

  if (self.head_init) then
    self.head_init = false
    commands.setHeadPan(self.pan,self.time)
  end
end
function head_MoveHead:finished()
  return (self:getTime() > self.time)
end

--------------------------------
-- now that we never change tilt
-- only one scan!

---------------------------------------------
-- Basic Sweep scani

CompTask('head_SweepScan',{
           init = true,
           maxPan = 2.0,
           period = 3.0,
           numSweeps = 1,
           direction = 1,
           -- internal params
           intDirection = 1,
           sweepCounter = 0
         })
function head_SweepScan:choose(as)
  local scan = head_DiscreteScan
  local numSteps = self.period / scan.stepTime + 1
  local stepSize = (2 * self.maxPan / numSteps) * 1.05 -- 1.05 fudge factor to make it end where we want

  if self.init then
    self.init = false
    self.sweepCounter = 0
    self.intDirection = self.direction
    return scan:set{dest = self.direction * self.maxPan,stepSize = stepSize, init=true}
  end

  if as:finished() then
    self.sweepCounter = self.sweepCounter + 1
    self.intDirection = -self.intDirection
    return scan:set{dest = self.intDirection * self.maxPan,stepSize = stepSize, init=true,skipFirstPause=true}
  end

  return as
end

function head_SweepScan:finished()
  return self.sweepCounter >= self.numSweeps
end


------------------------------------------------------------

CompTask('head_LookForBall')
function head_LookForBall:choose(as)
  robot_vision.doHighResBallScan = true
  return head_LookForObject:set{ object = core.WO_BALL }
end

function head_LookForBall:finished()
  return true
end

--------

-- try to look where object should be from localization
CompTask('head_LookAtObject',{
           object = 0
         })
function head_LookAtObject:choose(as)
  --UTdebug.log(20, "lookatObj time", self:getTime())
  local wo = world_objects:getObjPtr(self.object)
  return head_LookAtPoint:set{ dist = wo.distance, bear = wo.bearing,
                               elev = wo.elevation }
end

------------

CompTask('head_LookForObject',{
           object = 9,
           lfoState = 0,
         })

function head_LookForObject:choose(as)

  local wo = world_objects:getObjPtr(self.object)

  local object = self.object
  local seen = wo.seen

  -- if we see it, track using vision bearing and distance
  -- if not, try looking where it should be
  -- if that fails after some time, try scan

  if as == nil then
    as = task_NullTask:set()
  end

  -- seen, track object
  if (seen) then
    UTdebug.log(30, "Tracking Ball")
    self.lfoState = 1
    return head_TrackObject:set{ object = object }
  else
    if (self.lfoState == 0 or self.lfoState == 1) then
      -- first time without seeing ball
      UTdebug.log(30, "First time not seeing ball")
    end

    --UTdebug.log(20, "SATO, ", seen, self.lfoState)
    -- not seen
    -- do scan if we've looked and its not where we thought

    -- if its the ball, and we're not sure where it is... go straight to scan
    -- rather than try to look
    -- it's pretty cheap though, so err on the side of looking for the ball
    -- check ball sd relative to robot sd
    local me = world_objects:getObjPtr(robot_state.WO_SELF)
    local relSD = wo.sd - me.sd

    UTdebug.log(30, "ball abs sd", wo.sd, " rel sd", relSD)
    local sdThresh = 1500
    if (wo:isBall() and (relSD.x > sdThresh or relSD.y > sdThresh)) then
      UTdebug.log(30, "look for ball, high sd, skip look at point and go straight to scan")
      self.lfoState = 3
    end

    local lookAtPointTime = 0.6
    local lookAtObjectTask = head_LookAtObject

    if (self.lfoState == 3 or
        (as == lookAtObjectTask and as:getTime() > lookAtPointTime)) then
      UTdebug.log(30, "going into sweep")
      self.lfoState = 3
      if (as == head_SweepScan) then
        return as
      end
      -- start scanning in the direction the wo was last seen
      local direction
      if (wo.bearing > 0) then
        direction = 1
      else
        direction = -1
      end
      return head_SweepScan:set{maxPan = 2.09,
                                period = 3.0, --2.6,
                                numSweeps = 1,
                                direction = direction}
    else
      UTdebug.log(30, "looking for object based on expected position")
      self.lfoState = 2
      return lookAtObjectTask:set{ object = self.object }
    end
  end
end

-- look at the given point
PrimTask('head_LookAtPoint',{
           dist = 10,
           bear = 0,
           elev = 0,
           head_init = true
         })
function head_LookAtPoint:run()

  self.head_init = false

  local pan = self.bear;

  -- time should get smaller the longer we've run this method
  local targetTime = 0.35 - self:getTime()
  targetTime = math.max(0.05, targetTime)

  UTdebug.log(90, "LookAtPoint", self.dist, RAD_T_DEG*self.bear, self.elev, RAD_T_DEG*pan, targetTime)

  commands.setHeadPan(pan,targetTime)

end


--------
-- try to center object in image
PrimTask('head_TrackObject',{
           object = 0
         })

function head_TrackObject:run()

  local wo = world_objects:getObjPtr(self.object)

  if (wo.seen) then
    --print (vision_frame_info.frame_id, wo.seen)

    local panTrackSpeed = 0.05

    local panChange = panTrackSpeed * ((core.IMAGE_X / 2.0) - wo.imageCenterX)/core.IMAGE_FACTOR
    UTdebug.log(25, "tracking, object center, track speed, change", wo.imageCenterX, panTrackSpeed, panChange)

    commands.setHeadPan(panChange,0.75,true)
  else
    return head_LookAtObject:set{object = self.object}
  end
end

----------


PrimTask('head_DiscreteScan',{
           dest = 115 * DEG_T_RAD, -- how far left/right to pan
           stepSize = 26 * DEG_T_RAD,
           stepTime = 0.4,
           pauseTime = 0.2083,
           skipFirstPause = false,
           -- internal
           init = true,
           isFinished = false,
           isPaused = true,
         })
function head_DiscreteScan:run()
  if self.init then
    self.init = false
    self.isFinished = false
    self.isPaused = true
    self:resetTime()
  end

  local moveTime = self.stepTime - self.pauseTime

  if self.skipFirstPause or (self.isPaused and (self:getTime() > self.pauseTime)) then
    self.isPaused = false
    self.skipFirstPause = false
    -- set the new target
    local currPan = joint_angles[core.HeadPan]
    local diff = self.dest - currPan
    local dir = (diff / math.abs(diff))
    local target = currPan + self.stepSize * dir
    if (dir > 0) and (target > self.dest) then
      target = self.dest
    elseif (dir < 0) and (target < self.dest) then
      target = self.dest
    end
    commands.setHeadTilt()
    commands.setHeadPan(target,moveTime)
    if math.abs(diff) < 5 * DEG_T_RAD then
      self.isFinished = true
    end
    self:resetTime()
  end

  if not self.isPaused and (self:getTime() > moveTime) then
    self:resetTime()
    self.isPaused = true
  end
end

function head_DiscreteScan:finished()
  return self.isFinished
end

