require 'task'

module(...,package.seeall)

-- list of possible kicks
-- Kick Info class decleration
KickInfo = {}
KickInfo.__index = KickInfo
function KickInfo.create(switch,heading,distance,alignment,time,foot,isSlow,isSet)
  local tab = {} 
  setmetatable(tab,KickInfo)
  tab.switchable = switch
  tab.heading = heading
  tab.distance = distance
  tab.alignment = alignment
  tab.foot = foot
  tab.time = time
  tab.isSlow = isSlow -- kicks where we have to stop walking
  tab.isSet = isSet -- for variable kicks (gap, passes), if its set and valid
  return tab
end

KickAlignment = {}
KickAlignment.__index = KickAlignment
function KickAlignment.create(des_x, max_stop_x, des_y, min_stop_y, max_stop_y)
  local tab = {}
  setmetatable(tab,KickAlignment)
  tab.des_x = des_x
  tab.max_stop_x_err = max_stop_x
  tab.des_y = des_y
  tab.min_stop_y_err = min_stop_y
  tab.max_stop_y_err = max_stop_y
  return tab
end

-- des_x, max_stop_x, des_y, min_stop_y, max_stop_y
local fwdKickAlignment       = KickAlignment.create( 20, 20, 40, -20, 20)
local walkSideKickAlignment  = KickAlignment.create( 10, 20, -15, -15, 10) --(2.5,  5, -5, -10,  5)
local walkFrontKickAlignment = KickAlignment.create( 20, 10, 20, -20, 20)
local walkAngleKickAlignment = KickAlignment.create(10,  20, 0, -10,  10)
local angInKickAlignment     = KickAlignment.create( 16, 20, 37, -10, 5)  --15 17
local angOutKickAlignment    = KickAlignment.create( 19, 20, 50, -3, 10)
local sideKickAlignment      = angKickAlignment
local dribbleAlignment       = KickAlignment.create(45, -999, 0, -50, 50) -- desx not really used for now

-- setup indices to the passes
Passes = {}
Passes[2] = core.FwdPass2Kick
Passes[3] = core.FwdPass3Kick
Passes[4] = core.FwdPass4Kick

-- Each kick needs set some kick parameters
-- 0) Switchable, i.e. can we choose left or right leg at the last moment
-- 1) Heading of the kick (degrees)
-- 2) Distance, approximate length (mm)
-- 3) Desired Y, location of the ball (to the side) (mm)
-- 4) Execution Time, time to kick (seconds)

local fwdShort = 1500
local fwdMedium = 2250
local fwdLong = 3000
local fwdLeftwardAngle =  0.34 -- 20 deg now    0.5236 -- 30deg
local fwdRightwardAngle = -0.34 -- 20deg now    -0.5236 -- -30deg
local sideDist = 750
local fwdYOffset = 40.0
local angYOffset = 20.0

kickData = {}
-- short forward kicks
kickData[core.FwdShortStraightKick] = KickInfo.create(true, 0.0, fwdShort, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, false)
kickData[core.FwdShortLeftwardKick] = KickInfo.create(false, fwdLeftwardAngle, fwdShort, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, false)
kickData[core.FwdShortRightwardKick] = KickInfo.create(false, fwdRightwardAngle, fwdShort, fwdKickAlignment, 2.0, core.Kick_LEFT, true, false)

--kickData[core.FwdShortLeftwardInKick] = KickInfo.create(false, fwdLeftwardAngle, fwdShort, angInKickAlignment, 2.0, core.Kick_RIGHT, true, true)
--kickData[core.FwdShortRightwardInKick] = KickInfo.create(false, fwdRightwardAngle, fwdShort, angInKickAlignment, 2.0, core.Kick_LEFT, true, true)
--kickData[core.FwdShortLeftwardOutKick] = KickInfo.create(false, fwdLeftwardAngle, fwdShort, angOutKickAlignment, 2.0, core.Kick_LEFT, true, true)
--kickData[core.FwdShortRightwardOutKick] = KickInfo.create(false, fwdRightwardAngle, fwdShort, angOutKickAlignment, 2.0, core.Kick_RIGHT, true, true)

-- medium forward kicks
kickData[core.FwdMediumStraightKick] = KickInfo.create(true, 0.0, fwdMedium, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, true)
kickData[core.FwdMediumLeftwardInKick] = KickInfo.create(false, fwdLeftwardAngle, fwdMedium, angInKickAlignment, 2.0, core.Kick_RIGHT, true, true)
kickData[core.FwdMediumRightwardInKick] = KickInfo.create(false, fwdRightwardAngle, fwdMedium, angInKickAlignment, 2.0, core.Kick_LEFT, true, true)
kickData[core.FwdMediumLeftwardOutKick] = KickInfo.create(false, fwdLeftwardAngle, fwdMedium, angOutKickAlignment, 2.0, core.Kick_LEFT, true, true)
kickData[core.FwdMediumRightwardOutKick] = KickInfo.create(false, fwdRightwardAngle, fwdMedium, angOutKickAlignment, 2.0, core.Kick_RIGHT, true, true)

-- long forward kicks
kickData[core.FwdLongStraightKick] = KickInfo.create(true, 0.0, fwdLong, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, true)
kickData[core.FwdLongLeftwardInKick] = KickInfo.create(false, fwdLeftwardAngle, fwdLong, angInKickAlignment, 2.0, core.Kick_RIGHT, true, true)
kickData[core.FwdLongRightwardInKick] = KickInfo.create(false, fwdRightwardAngle, fwdLong, angInKickAlignment, 2.0, core.Kick_LEFT, true, true)
kickData[core.FwdLongLeftwardOutKick] = KickInfo.create(false, fwdLeftwardAngle, fwdLong, fwdKickAlignment, 2.0, core.Kick_LEFT, true, true)
kickData[core.FwdLongRightwardOutKick] = KickInfo.create(false, fwdRightwardAngle, fwdLong, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, true)

-- aiming for goal gaps
kickData[core.FwdLongLargeGapKick] = KickInfo.create(true, 0.0, fwdLong, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, false)
kickData[core.FwdLongSmallGapKick] = KickInfo.create(true, 0.0, fwdLong, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, false)

-- passing to teammates
kickData[core.FwdPass4Kick] = KickInfo.create(true, 0.0, fwdLong, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, false)
kickData[core.FwdPass3Kick] = KickInfo.create(true, 0.0, fwdLong, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, false)
kickData[core.FwdPass2Kick] = KickInfo.create(true, 0.0, fwdLong, fwdKickAlignment, 2.0, core.Kick_RIGHT, true, false)

-- side kicks
kickData[core.LeftwardSideKick] = KickInfo.create(false, 1.397, sideDist, sideKickAlignment, 2.0, core.Kick_RIGHT, true, false)
kickData[core.RightwardSideKick] = KickInfo.create(false,-1.397, sideDist, sideKickAlignment, 2.0, core.Kick_LEFT, true, false)

-- really fast kick
kickData[core.FastKick] = KickInfo.create(true,0.0, 650, fwdKickAlignment, 0.75, core.Kick_RIGHT, false, false)
kickData[core.Dribble] = KickInfo.create(true, 0.0, 250.0, dribbleAlignment, 0.5, core.Kick_RIGHT, false, true)

--function KickInfo.create(switch,heading,distance,y,time,foot,isSet)
-- kicking from the walk engine
kickData[core.WalkKickFront] = KickInfo.create(true,0.0,1250.0,walkFrontKickAlignment,0.25,core.Kick_RIGHT,false, true)
kickData[core.WalkKickLeftwardSide] = KickInfo.create(false,DEG_T_RAD * 90,500.0,walkSideKickAlignment,0.25,core.Kick_RIGHT,false, true)
kickData[core.WalkKickRightwardSide] = KickInfo.create(false,DEG_T_RAD * -90,500.0,walkSideKickAlignment,0.25,core.Kick_LEFT,false, true)
kickData[core.WalkKickLeftward] = KickInfo.create(false,DEG_T_RAD * 45,1000.0,walkAngleKickAlignment,0.25,core.Kick_RIGHT,false, true)
kickData[core.WalkKickRightward] = KickInfo.create(false,DEG_T_RAD * -45,1000.0,walkAngleKickAlignment,0.25,core.Kick_LEFT,false, true)

function checkKicks()
  -- check for kicks we haven't set up corrects
  for i = 0, core.NUM_KICKS-1 do
    if kickData[i] == nil then
      UTdebug.log(0,'*** KICK DATA MISSING FOR:',luaC:getString(core.kickNames,i))
      kickData[i] = KickInfo.create(true,0,0,fwdKickAlignment,10.0,core.Kick_LEFT,true,false)
    end
  end
end

CompTask('kicks_ExecuteKick',{
           init = true,
           kickRunning = false,
           desiredDistance = 3000,
           desiredHeading = 0.0,
           postKick = false,
           kickChoice = core.FwdLongStraightKick,
           lastChoice = core.FwdLongStraightKick
         })
function kicks_ExecuteKick:choose(as)
  -- set some ball stuff for all kicks
  local ball = world_objects:getObjPtr(core.WO_BALL)
  kick_request.ball_seen_ = ball.seen
  kick_request.ball_image_center_x_ = ball.imageCenterX
  kick_request.ball_image_center_y_ = ball.imageCenterY
  kick_request.ball_rel_x_ = localizationC.filtered_close_ball_.x
  kick_request.ball_rel_y_ = localizationC.filtered_close_ball_.y

  commands.last_arm_command = vision_frame_info.seconds_since_start
  if (self.kickChoice == core.WalkKickFront) or (self.kickChoice == core.WalkKickLeftwardSide) or (self.kickChoice == core.WalkKickRightwardSide) or (self.kickChoice == core.WalkKickLeftward) or (self.kickChoice == core.WalkKickRightward)  then
    if as == nil then
      self.kickRunning = true
    else
      self.kickRunning = as.kickRunning
    end
    return kicks_WalkKick:set{kickType=self.kickChoice,desiredHeading=self.desiredHeading,desiredDistance=self.desiredDistance}
  elseif (self.kickChoice == core.FastKick) then
    return oldkicks.kicks_FastPoseKick:set{}
  elseif ((self.kickChoice == core.LeftwardSideKick) or (self.kickChoice == core.RightwardSideKick)) then
    return oldkicks.kicks_QuickSideKick:set{kickType=self.kickChoice}
  --elseif (self.kickChoice == core.FwdStraightKickCOM) or (self.kickChoice == core.FwdStraightKickCOMSpline) then
    -- handled below
  --else
    --return oldkicks.kicks_QuickKick:set{kickType=self.kickChoice,desiredKickHeading=self.desiredHeading,desiredKickDistance=self.desiredDistance}
  else
    -- handled below for Katie's kick
  end

  if (self.init) then 
    self.kickRunning = true
    kick_request.kick_running_ = true
    walk_request:stand()

    -- set initial kick request
    if (kickData[self.kickChoice].switchable) then
      kick_request:set(core.Kick_STRAIGHT, core.Kick_SWITCHABLE, self.desiredHeading, self.desiredDistance)
    else
      kick_request:set(core.Kick_STRAIGHT, kickData[self.kickChoice].foot, self.desiredHeading, self.desiredDistance)
    end

    --if (walk_info.walk_is_active_) then
    --  commands.stand()
    --  self.init = true
    --  return head_LookForBall:set()
    --end
    self.init = false
    self.lastChoice = self.kickChoice
    self:resetTime()
  elseif self:getTime() < 0.15 then
    self.kickRunning = true
    --if not(kick_request.kick_running_) then
      --UTdebug.log(0,'Would have aborted kick')
    --end
  else
    self.kickRunning = kick_request.kick_running_
  end

  if (self.kickRunning) then 
    walk_request:noWalk()
  end

  UTdebug.log(10,"kick running", kick_request.kick_running_, self.kickRunning)

  kick_request.allow_correction_walk_ = false --true
  --UTdebug.log(0,"vision ball loc:",ball.imageCenterX,ball.imageCenterY)

  local offset = getCamOffsets()
  kick_request.camera_offset_x_ = offset.pix_offset_x
  kick_request.camera_offset_y_ = offset.pix_offset_y
  kick_request.camera_tilt_offset_ = 0 

  if (odometry.didKick) then
    self.postKick = true
  end

  -- always set kick request in case its changing
  -- up until actual kick execution
  if (not self.postKick) then
    if (kickData[self.kickChoice].switchable) then
      kick_request.kick_leg_ = core.Kick_SWITCHABLE
    else
      kick_request.kick_leg_ = kickData[self.kickChoice].foot
    end
    kick_request.desired_angle_ = self.desiredHeading
    kick_request.desired_distance_ = self.desiredDistance
    self.lastChoice = self.kickChoice
  end

  if (self.postKick) then
    --commands.setWalkVelocity(0,0,0)
    -- set head to look for ball
    -- look at point for 0.75 seconds, then call look for ball
    if (as == head_LookForBall or (as == head_LookAtPoint and as:getTime() > 1.0)) then
      return head_LookForBall:set()
    end
    
    return head_LookAtPoint:set{dist = self.desiredDistance, bear = self.desiredHeading}
  end

  return task_NullTask:set()
end

function kicks_ExecuteKick:finished()
  return not self.kickRunning
end

CompTask('kicks_WalkKick',{
    init = true,
    kickRunning = false,
    desiredDistance = 1000,
    desiredHeading = 0,
    kickType = core.WalkKickFront,
    leftFoot = true
  })
function kicks_WalkKick:choose(as)
  if self.init then
    local ball = world_objects:getObjPtr(core.WO_BALL)
    self.init = false
    self.kickRunning = true
    if self.kickType == core.WalkKickFront then
      self.leftFoot = localizationC.filtered_close_ball_.y > 0
    elseif (self.kickType == core.WalkKickLeftwardSide) or (self.kickType == core.WalkKickLeftward) then
      self.leftFoot = false
    elseif (self.kickType == core.WalkKickRightwardSide) or (self.kickType == core.WalkKickRightward) then
      self.leftFoot = true
    else
      UTdebug.log(0,'Invalid kick type sent to kicks_WalkKick:',self.kickType,luaC:getString(core.kickNames,self.kickType))
      self.leftFoot = true
    end
  end
  if self.kickRunning then
    commands.setWalkVelocity(0,0,0,false)
    walk_request:setKick(self.desiredDistance,self.desiredHeading,self.leftFoot)
    walk_request.new_command_ = true
  end
  if (odometry.didKick) or self:getTime() > 1.0 then
    self.kickRunning = false
  end
  if true then
    return head_LookForBall:set()
  end
end

function kicks_WalkKick:finished()
  return not self.kickRunning
end



CompTask('kicks_QuickStepKick',{
           state = 0,
           isLeft = true,
           done = false
  })
function kicks_QuickStepKick:choose(as)

  -- reset walk request
  walk_request:wait()

  if (self.state == 0) then
    commands.stand()
    if (odometry.standing) then
      self.state = 1
      self:resetTime()
    end
  end

  if self.state == 1 then
    if (self.isLeft) then
      --speech:say('Quick Left')
    else
      --speech:say('Quick Right')
    end
    self.state = 2
    self:resetTime()
    commands.setWalkMode(core.WalkMode_KICK)
    walk_request:setStep(self.isLeft, 0.1, 0.0, 0)
  end

  if (self.state == 2 and self:getTime() > 1.5) then
    commands.setALWalkParameters(cfgWalkKickBack)
    self.state = 3
    self:resetTime()
    walk_request:setStep(self.isLeft, 0.00, 0.0, 0)

    -- odometry update
    odometry.didKick = true
    odometry.kickHeading = 0
    odometry.kickVelocity = 500.0;

  end

  if (self.state == 3 and self:getTime() > 1.5) then
    self.done = true
  end


  return head_LookForBall:set()
end

function kicks_QuickStepKick:finished()
  return self.done
end


CompTask('kicks_ExecuteWalkEngineKick',{
           init = true,
           kickRunning = false,
           desiredDistance = 2000,
           desiredHeading = 0.0,
           kick_complete = false
         })
function kicks_ExecuteWalkEngineKick:choose(as)
  if (self.init) then
    -- get ball location
    local ball = world_objects:getObjPtr(core.WO_BALL)

    -- set kick request
    commands.setWalkVelocity(0.5,0,0,false)
    walk_request:setKick(2000, 0, ball.bearing > 0)

    self.init = false
  end
    
  UTdebug.log(10, "Execute walk engine kick", walk_request.perform_kick_, walk_request.kick_with_left_)

  if (not walk_request.perform_kick_ and not self.kick_complete) then
    self.kick_complete = true
    self:resetTime()
  end

  return task_NullTask:set()
end

function kicks_ExecuteWalkEngineKick:finished()
  return self.kick_complete and self:getTime() > 1.5
end



-- Todd: fake kick for behavior debug
CompTask('kicks_FakeKick',{
           init = true,
           desiredDistance = 2000,
           desiredHeading = 0
         })
function kicks_FakeKick:choose(as)
  if (self.init) then
    -- Todd: TODO: how do we access kick types and foot from lua???
    kick_request:set(1,2, self.desiredHeading, self.desiredDistance)
    self.init = false
  end
  return task_NullTask:set()
end

function kicks_FakeKick:finished()
  return self:getTime() > 2.0
end
