#ifndef IMAGEBLOCK_95BEQPL8
#define IMAGEBLOCK_95BEQPL8

#include "MemoryBlock.h"
#include <common/RobotInfo.h>
#include <boost/interprocess/offset_ptr.hpp>
#include <iostream>

struct ImageBlock : public MemoryBlock {
public:
  // private arrays used for logging and for sim
  unsigned char img_top_local_[RAW_TOP_IMAGE_SIZE];
  unsigned char img_bottom_local_[RAW_BOTTOM_IMAGE_SIZE];
public:
  ImageBlock()  {
    header.version = 2;
    header.size = sizeof(ImageBlock);
    img_top_ = NULL;
    img_bottom_ = NULL;
  }

  bool isFromLog() {
    bool isTopFromLog = img_top_ == boost::interprocess::offset_ptr<unsigned char>(&(img_top_local_[0]));
    bool isBottomFromLog = img_bottom_ == boost::interprocess::offset_ptr<unsigned char>(&(img_bottom_local_[0]));
    return isTopFromLog && isBottomFromLog;
  }

  void preWrite() {
    // copy the image pointer to memory
    if (!isFromLog()) {
      memcpy(img_top_local_, img_top_.get(), RAW_TOP_IMAGE_SIZE);
      memcpy(img_bottom_local_, img_bottom_.get(), RAW_BOTTOM_IMAGE_SIZE);
    }
  }

  void postRead() {
    // if we're reading from a block that had null pointers (no images)
    // then keep them that way
    if (img_top_ != NULL) img_top_ = img_top_local_;
    if (img_bottom_ != NULL) img_bottom_ = img_bottom_local_;
  }

  boost::interprocess::offset_ptr<unsigned char> img_top_;
  boost::interprocess::offset_ptr<unsigned char> img_bottom_;
};

#endif /* end of include guard: IMAGEBLOCK_95BEQPL8 */
