#include "GetupModule.h"

#include <memory/FrameInfoBlock.h>
#include <memory/JointCommandBlock.h>
#include <memory/WalkRequestBlock.h>
#include <memory/OdometryBlock.h>
#include <memory/BodyModelBlock.h>
#include <memory/JointBlock.h>

#define FIRST_FRAME_GRADUAL

const float interpFastFactor = 0.0;   //0.0 means no fastFactor for interp
const float WAIT_PREPAREFALL= 0.30;
const float LOST_STIFFTIME = 0.25; 
const bool CHECKER = false;

GetupModule::GetupModule():
  overHeatJoint(false),last_diffT(0.0),enterPrepare(false),numCrosses(0),loseStiffnessTime(LOST_STIFFTIME),waitPrepareFallTime(WAIT_PREPAREFALL),numPrepare(0),frameWait(0),enterStatusChecker(0)
{
  Joints_beforePrepare =  vector<float>(NUM_JOINTS,VOID_NUM*DEG_T_RAD);
}

void GetupModule::initGetup() {
  getUpSide = Getup::UNKNOWN;
  initSpecialMotion();
  initialized_MotionSequence=false;
  numCrosses = 0;
  numPrepare = 0;
  frameWait=0;
  enterStatusChecker = 0;
  loseStiffnessTime=LOST_STIFFTIME;
  waitPrepareFallTime = WAIT_PREPAREFALL;
}

bool GetupModule::isGettingUp()
{
  return isDoingSpecialMotion();
}

void GetupModule::processGetup() {

  processed_joint_commands_->send_stiffness_ = false;

  // set getting up odometry
  if (isGettingUp()){
    // which way are we getting up?
    odometry_->getting_up_side_ = getUpSide;
  } else {
    // no getup
    odometry_->getting_up_side_ = Getup::NONE;
    // set getup side back to unknown for the next getup
    getUpSide = Getup::UNKNOWN;
  }

  // std::cout<<"Start"<<getup_state_<<std::endl;

  // state 0: turn off stiffness
  if (specialmotion_state_== 0) {
    specialmotion_pose_count_ = 0;
    // stiffness should already be off from lua command
    //processed_joint_commands_->send_stiffness_ = true;
    //processed_joint_commands_->stiffness_time_ = 10;
    //for (int i = 0; i < NUM_JOINTS; i++)
    //  processed_joint_commands_->stiffness_[i] = 0.0;
    loseStiffnessTime = LOST_STIFFTIME;
    waitPrepareFallTime = WAIT_PREPAREFALL;
    numPrepare = 0;
    frameWait=0;
    enterStatusChecker = 0;
    if (walk_request_->getup_from_keeper_dive_) {
      specialmotion_state_=2;
    } else {
      specialmotion_state_=1;
    }
    specialmotion_state_time_ = frame_info_->seconds_since_start;
    last_diffT=0.0;
  }


  // state 1: wait for us to finish the fall (a fraction of second)
  if (specialmotion_state_== 1) {
    enterStatusChecker = 0;
    double diff = frame_info_->seconds_since_start - specialmotion_state_time_;
    //  cout << "FALL side " << walk_request_->tilt_fallen_counter_ << " roll: " << walk_request_->roll_fallen_counter_ << endl;
    waitPrepareFallTime = (fabs(walk_request_->roll_fallen_counter_) > 50)?0.2:0.0 + WAIT_PREPAREFALL;
    loseStiffnessTime = (fabs(walk_request_->roll_fallen_counter_) > 50)?0.2:0.0 + LOST_STIFFTIME;
    if(diff<=loseStiffnessTime) 
    {
       loseStiffness();
       enterPrepare = false;
    }
    else if (diff<=loseStiffnessTime + waitPrepareFallTime )
    {
       // adjustment before fall: fall back, in case arm under body
        if (walk_request_->tilt_fallen_counter_  < 0||fabs(walk_request_->roll_fallen_counter_) > 150)
        {
               prepareFall(diff-loseStiffnessTime);
        }
    }
    else{
      specialmotion_pose_count_ = 0;
      processed_joint_commands_->send_stiffness_ = true;
      processed_joint_commands_->stiffness_time_ = 10;
      for (int i = 0; i < NUM_JOINTS; i++)
        processed_joint_commands_->stiffness_[i] = 1.0;
      specialmotion_state_ = 3;  //added just ignore the cross part
      specialmotion_state_time_ = frame_info_->seconds_since_start;

      if (numPrepare < 3  && fabs(walk_request_->roll_fallen_counter_) > 3) {
          cout << "FALL ON SHOULDER??TIME EXTENDED, REDO PREPARE:  " << walk_request_->tilt_fallen_counter_ << " roll: " << walk_request_->roll_fallen_counter_ << endl;
          specialmotion_state_time_ = frame_info_->seconds_since_start;
          specialmotion_state_ = 1;
         // loseStiffnessTime+=0.5;
          waitPrepareFallTime += fabs(walk_request_->roll_fallen_counter_)/40;
          numPrepare += 1;
          return;
        }
    }
  }

  // state 2: do the cross, if not fallen on side
  else if (specialmotion_state_ == 2) {
    enterStatusChecker = 0;
    cross();   
    last_sent_state_ = 0;
    //cout << "Tilt fall: " << walk_request_->tilt_fallen_counter_ << " roll: " << walk_request_->roll_fallen_counter_ << " # crosses: " << numCrosses << endl << flush;
  }

  // state 3: start getting up
  else if (specialmotion_state_ == 3) {

    if(!initialized_MotionSequence)
    {
        overHeatJoint=false;
        //detection for joint heat...
        for(int i=0; i< NUM_JOINTS; i++)
          {
            if(sensors_->joint_temperatures_[i]>70)
              {
                #ifdef JOINT_HEAT_FACTOR
                heatFactors[i]=(sensors_->joint_temperatures_[i]-70)/50.0*JOINT_HEAT_FACTOR;
                #else
                heatFactors[i]=(sensors_->joint_temperatures_[i]-70)/50.0*0.4;
                overHeatJoint=true;  //not sure yet
                #endif
                std::cout<<"OverHeat "<<getJointName(Joint(i))<<"="<<heatFactors[i]<<std::endl<<std::flush;
              }
          }
        //cout << "Tilt fall: " << walk_request_->tilt_fallen_counter_ << " roll: " << walk_request_->roll_fallen_counter_ << " # crosses: " << numCrosses << endl << flush;

        if (walk_request_->tilt_fallen_counter_ < -5){
          odometry_->getting_up_side_ = Getup::BACK;
          motionID = "standUpBackNaoH";
          getUpSide = Getup::BACK;
        }
        else if (walk_request_->tilt_fallen_counter_ > 5){
          odometry_->getting_up_side_ = Getup::FRONT;
          motionID = "standUpFrontNaoH";
          getUpSide = Getup::FRONT;
        }
        else if (numCrosses > 0){
          // dont know which way to go and already did a cross.. just pick one
          if (walk_request_->tilt_fallen_counter_ < -2){
            odometry_->getting_up_side_ = Getup::BACK;
            motionID = "standUpBackNaoH";
            getUpSide = Getup::BACK;
          }
          else if(walk_request_->tilt_fallen_counter_ > 2){
            odometry_->getting_up_side_ = Getup::FRONT;
            motionID = "standUpFrontNaoH";
            getUpSide = Getup::FRONT;
          }
          else 
          {
            std::cout<<"111 STILL NOT KNOW WHICH DIRECTION FALLEN : after cross"<<endl;   
            specialmotion_state_time_ = frame_info_->seconds_since_start;
            frameWait++;
            if(frameWait<30)
                return;
            else
            {
                 odometry_->getting_up_side_ = Getup::BACK;
                 motionID = "standUpBackNaoH";
                 getUpSide = Getup::BACK;
                std::cout<<"DEFAULT GETUP MOTION: "<<motionID;
                initialize_MotionSequence();
                std::cout<<"finishInitialized"<<std::endl;
                initialized_MotionSequence=true;
            }
          }
        }
        else if (fabs(walk_request_->roll_fallen_counter_) > 2) {
          cout << "DONT KNOW WHICH SIDE YET, tilt: " << walk_request_->tilt_fallen_counter_ << " roll: " << walk_request_->roll_fallen_counter_ << endl;
          specialmotion_state_time_ = frame_info_->seconds_since_start;
          specialmotion_state_ = 2;
          cross();   //TODO: SPECIAL CASE : CROSS
          last_sent_state_ = 0;
          return;
        }
        else {
          // do nothing
          std::cout<<"222 STILL NOT KNOW WHICH DIRECTION FALLEN : after cross"<<endl;   
          specialmotion_state_time_ = frame_info_->seconds_since_start;
          frameWait++;
          if(frameWait<10)
                return;
          else
          {
                 odometry_->getting_up_side_ = Getup::BACK;
                 motionID = "standUpBackNaoH";
                 getUpSide = Getup::BACK;
                std::cout<<"DEFAULT GETUP MOTION: "<<motionID;
                initialize_MotionSequence();
                std::cout<<"finishInitialized"<<std::endl;
                initialized_MotionSequence=true;
          }
        }

        //  if(overHeatJoint)
        std::cout<<"GETUP MOTION: "<<motionID;
        initialize_MotionSequence();
        std::cout<<"finishInitialized"<<std::endl;
        initialized_MotionSequence=true;
      }

    else
    {
        // std::cout << "continue getup " << std::endl;// << std::flush;
        double diff = frame_info_->seconds_since_start - specialmotion_state_time_;
        if (diff > waitFinishedTime){
            std::cout << " Getup complete" << std::endl;// << std::flush;
            specialmotion_state_ = 4;
        }
  
        else
        {
            /** might need to check?  as in state 3*/
            if (walk_request_->tilt_fallen_counter_ < 0) {
                processed_joint_commands_->send_back_standup_ = true;
                doing_back_getup_ = true;
            }
            else
            {
                //   processed_joint_commands_->send_front_standup_ = true;
                doing_front_getup_ = true;
            }
            executeMotionSequence();

            ///NEW : check whether we are in correct position in the middle of get up, if not restart motion. So we don't have to wait the failed motion to finish.
            if(CHECKER && motionID == "standUpFrontNaoH" &&diff > waitFinishedTime*0.7*0.001 && enterStatusChecker == 0 )
            {
                if(fabs(walk_request_->roll_fallen_counter_) !=0 || fabs(walk_request_->tilt_fallen_counter_)!= 0)
                   
                if(fabs(walk_request_->roll_fallen_counter_) > 35 || fabs(walk_request_->tilt_fallen_counter_) > 35)
                {
                     //still on the ground or on arm/side
                    initialized_MotionSequence = false;
                    enterStatusChecker = 1; 
                    specialmotion_state_time_ = frame_info_->seconds_since_start - 0.6;  //restart from 0.6 second of the motion
                    cout<<"FAILED_RESTART: FRONT GETUP "<< walk_request_->tilt_fallen_counter_ << " roll: " << walk_request_->roll_fallen_counter_ << endl;
                    //restart the entire get up before finish all.
                }            
            }
             if(CHECKER && motionID == "standUpBackNaoH" &&diff > waitFinishedTime*0.6*0.001 && enterStatusChecker == 0 )
            {
                if(fabs(walk_request_->roll_fallen_counter_) > 35 || fabs(walk_request_->tilt_fallen_counter_) > 35)
                {
                     //still on the ground or on arm/side
                    initialized_MotionSequence = false;
                    enterStatusChecker = 1; 
                    specialmotion_state_time_ = frame_info_->seconds_since_start - 0.2; //restart from 0.2 seconds of the motion.
                    cout<<"FAILED_RESTART: BACK GETUP "<< walk_request_->tilt_fallen_counter_ << " roll: " << walk_request_->roll_fallen_counter_ << endl;
                    //restart the entire get up before finish all.
                }            
            
            }
            //END DOING MOTION

        } 
    }
  }

  // state 4: wait for getup
  else if (specialmotion_state_ == 4){
    specialmotion_state_ = 5;
    specialmotion_state_time_ = frame_info_->seconds_since_start;
    enterStatusChecker = 0;
  }

  // state 5: wait for it to be stable
  else if (specialmotion_state_ == 5){
    double diff = frame_info_->seconds_since_start - specialmotion_state_time_;
    numCrosses = 0;
    if (diff > 0.5)
      specialmotion_state_ = 6;
  }
  else
    {
      std::cout<<"Wrong state number: "<< specialmotion_state_<<std::endl<<std::flush;
    }


}

void GetupModule::loseStiffness()
{
  std::cout<<"loseStiffness"<<std::flush;
  processJointHardness(1.0);
}

void GetupModule::prepareFall(float diffT)
{
  if(!enterPrepare)
    {
      for(int i=0;i<NUM_JOINTS;i++)
        {
          Joints_beforePrepare[i]=joint_angles_->values_[i];
        }
      enterPrepare=true;
    }
  float interp=1.0;
  if(waitPrepareFallTime<=0)
      waitPrepareFallTime= WAIT_PREPAREFALL;
#ifdef FIRST_FRAME_GRADUAL
  interp=(diffT)/waitPrepareFallTime;
#else
#endif
  interp*=1.0+interpFastFactor*(1.0-diffT/waitPrepareFallTime);

  float prepareAngles[NUM_JOINTS];
  for(int i=0;i<NUM_JOINTS;i++)
    {
          prepareAngles[i]=VOID_NUM*DEG_T_RAD;//prepareAngles[i]*DEG_T_RAD;
      //   if(i==LShoulderPitch||i==RShoulderPitch)  prepareAngles[i]=-2.5;
    }
  prepareAngles[HeadPitch] = ((DEG_T_RAD * -15)-Joints_beforePrepare[HeadPitch])*interp+Joints_beforePrepare[HeadPitch];//-30

  prepareAngles[LShoulderPitch] = ((DEG_T_RAD * -46)-Joints_beforePrepare[LShoulderPitch])*interp+Joints_beforePrepare[LShoulderPitch];
  prepareAngles[LShoulderRoll] = ((DEG_T_RAD * 32)-Joints_beforePrepare[LShoulderRoll])*interp+Joints_beforePrepare[LShoulderRoll];
  prepareAngles[LElbowYaw] = ((DEG_T_RAD * 80)-Joints_beforePrepare[LElbowYaw])*interp+Joints_beforePrepare[LElbowYaw]; //-85
  prepareAngles[LElbowRoll] = ((DEG_T_RAD * -50)-Joints_beforePrepare[LElbowRoll])*interp+Joints_beforePrepare[LElbowRoll]; //-40 before -20then
  prepareAngles[RShoulderPitch] = ((DEG_T_RAD * -46)-Joints_beforePrepare[RShoulderPitch])*interp+Joints_beforePrepare[RShoulderPitch];
  prepareAngles[RShoulderRoll] = ((DEG_T_RAD * 32)-Joints_beforePrepare[RShoulderRoll])*interp+Joints_beforePrepare[RShoulderRoll];
  prepareAngles[RElbowYaw] = ((DEG_T_RAD * 80)-Joints_beforePrepare[RElbowYaw])*interp+Joints_beforePrepare[RElbowYaw];
  prepareAngles[RElbowRoll] = ((DEG_T_RAD * -50)-Joints_beforePrepare[RElbowRoll])*interp+Joints_beforePrepare[RElbowRoll];
  //-116 40 25 0      -116 12 25 -53

   //  std::cout<<diffT<<"ArmCommand"<<interp<<":L"<< prepareAngles[LShoulderPitch]<<","<< prepareAngles[LShoulderRoll]<<","<< prepareAngles[LElbowYaw]<<","<< prepareAngles[LElbowRoll]<<std::endl;

  processJointArray(diffT-last_diffT,prepareAngles);
 //processJointArray(0,prepareAngles);
  processed_joint_commands_->send_stiffness_ = true;
  processed_joint_commands_->stiffness_time_ = 0;
  for (int i = 0; i < NUM_JOINTS; i++)
    processed_joint_commands_->stiffness_[i] = 0;
  processed_joint_commands_->stiffness_[HeadPitch] = 1.0;
  processed_joint_commands_->stiffness_[LShoulderPitch] = 1.0;
  processed_joint_commands_->stiffness_[LShoulderRoll] = 1.0;
  processed_joint_commands_->stiffness_[LElbowYaw] = 1.0;
  processed_joint_commands_->stiffness_[LElbowRoll] = 1.0;
  processed_joint_commands_->stiffness_[RShoulderPitch] = 1.0;
  processed_joint_commands_->stiffness_[RShoulderRoll] = 1.0;
  processed_joint_commands_->stiffness_[RElbowYaw] = 1.0;
  processed_joint_commands_->stiffness_[RElbowRoll] = 1.0;

  last_diffT=diffT;

#ifdef ANGLECOMMAND_CHECK_TURNOFF
  // cout<<"check the joint command--prepare <Getup>"<<endl;
  /** doing in all the joint command
      for(int i=0; i< NUM_JOINTS; i++)
      {
      //  cout<<"["<<i<<"]"<<(joint_angles_->values_[i]-prepareAngles[i])<<" ,";
      if(prepareAngles[i]!=VOID_NUM*DEG_T_RAD&&joint_angles_->values_[i]-prepareAngles[i]>ANGLECOMMAND_CHECK_TURNOFF)
      cout<<"Warning for Turn off>>Command!"<<i<<"current= " <<joint_angles_->values_[i]<<" request: "<<prepareAngles[i]<<endl;
      }
  */
#else
#endif

}

void GetupModule::cross() {
  double diff = frame_info_->seconds_since_start - specialmotion_state_time_;
  if (diff>0.9) {
    specialmotion_state_= 3; // go to 3 after cross
    specialmotion_state_time_ = frame_info_->seconds_since_start - 0.1001;
    numCrosses++;
    cout<<"END CROSS!!!!!!!!!!"<<endl;
    return;
  }
  // reset fallen counter, so we judge what side we're on after cross
  if (diff < 0.1) {
   // walk_request_->tilt_fallen_counter_ = 0;
  }
  if (diff < 0.001) {
    specialmotion_pose_count_ = 0;
    processed_joint_commands_->send_stiffness_ = true;
    processed_joint_commands_->stiffness_time_ = 10;
    for (int i = 0; i < NUM_JOINTS; i++)
      processed_joint_commands_->stiffness_[i] = 1.0;
  }

  if (diff > 0.15) 
  {  
     return; }

  float angles[NUM_JOINTS];
  for (int i=0; i<NUM_JOINTS; i++) {
    angles[i] = 0;
  }

  angles[HeadPitch] = DEG_T_RAD*30;
  angles[LShoulderRoll] = 1.5708;
  angles[RShoulderRoll] = 1.5708;
  angles[LShoulderPitch] = -1.5708;
  angles[RShoulderPitch] = -1.5708;
  // walk arms
  //angles[LShoulderPitch] = DEG_T_RAD * -96; // -116 //moving to make it farther outside the body
  //angles[LShoulderRoll] = DEG_T_RAD * 8;
  angles[LElbowYaw] = DEG_T_RAD * 25;
  angles[LElbowRoll] = DEG_T_RAD * -53;
  //angles[RShoulderPitch] = DEG_T_RAD * -96; // -116 //moving to make it farther outside the body
  //angles[RShoulderRoll] = DEG_T_RAD * 8;
  angles[RElbowYaw] = DEG_T_RAD * 25;
  angles[RElbowRoll] = DEG_T_RAD * -53;

  processJointArray(500,angles);
}

/**
   void GetupModule::backStandup() {
   doing_back_getup_ = true;   ///old;
   }

   void GetupModule::frontStandup() {
   doing_front_getup_ = true;   /// old
   }

   void GetupModule::backNBitesStandup() {
   /// old
   }
*/

