#ifndef SPECIAL_MOTION_MODULE_H
#define SPECIAL_MOTION_MODULE_H

#include <string>
#include <iostream>
#include <Module.h>
#include <common/RobotInfo.h>
#include <vector>
#include <motion/SpecialMotionParser.h>
#include <memory/FrameInfoBlock.h>
#include <memory/JointCommandBlock.h>
#include <memory/WalkRequestBlock.h>
#include <memory/OdometryBlock.h>
#include <memory/BodyModelBlock.h>
#include <memory/JointBlock.h>
#include <memory/SensorBlock.h>


using namespace std;

class FrameInfoBlock;
class WalkRequestBlock;
class JointCommandBlock;
class OdometryBlock;
class JointBlock;
class SensorBlock;

class SpecialMotionModule: public Module {
 public:
  SpecialMotionModule();
  void initialize(const string _motionID);
  void initialize(bool & _doing_specialmotion_, const string _motionID);
  void specifyMemoryDependency();
  void specifyMemoryBlocks();

  void processSpecialMotion();
  bool isDoingSpecialMotion() { return (specialmotion_state_ != 0 && specialmotion_state_ != 6); };
  void initSpecialMotion();

 //protected:
  FrameInfoBlock *frame_info_;
  JointCommandBlock* processed_joint_commands_;
  WalkRequestBlock* walk_request_;
  OdometryBlock* odometry_;
  JointBlock *joint_angles_;
  SensorBlock *sensors_;

    void processJointArray(float time, float angles[NUM_JOINTS]);
    void processJointHardness(float time,float stiffness[NUM_JOINTS]);
    void processJointHardness(float time);
    void executeMotionSequence();
    void initialize_MotionSequence();
    void initialize_MotionSequence(string motionName);
    void loadMotionFile();
    void addMotionEnd2WalkStart();
    void addMotionStart();
//   int jointMapping[NUM_JOINTS];

   float waitFinishedTime;
   string motionID;
   string teamID;
   bool biteCMD;
   bool biteCMDcurr;

//  bool doing_back_specialmotion_;
  bool doing_specialmotion_;
  int specialmotion_state_;
  double specialmotion_state_time_; 
  int specialmotion_pose_count_;
  int last_sent_state_;
 
  vector<SpecialMotion> MotionSequence;
  vector<float> times;
  vector<float> heatFactors;
  bool initialized_MotionSequence;
  float VEL;
};

#endif /* end of include guard: GETUP_MODULE */
