#ifndef BALLDETECTOR_H
#define BALLDETECTOR_H

#include <list>
#include <memory/TextLogger.h>
#include <memory/RobotVisionBlock.h>
#include <memory/BodyModelBlock.h>
#include <memory/WorldObjectBlock.h>
#include <memory/FrameInfoBlock.h>
#include <memory/RobotStateBlock.h>
#include <common/Field.h>
#include <memory/ImageBlock.h>
#include <vision/TransformationProvider.h>
#include <vision/BlobDetector.h>
#include <memory/TextLogger.h>
#include <constants/VisionConstants.h>
#include <vision/Macros.h>
#include <vision/structures/BallCandidate.h>
#include <math/Point.h>
#include <vision/structures/HorizonLine.h>

class BallDetector{
 public:
  BallDetector(WorldObjectBlock*&, BodyModelBlock*&, RobotVisionBlock*&, FrameInfoBlock*&, RobotStateBlock*&, Vision::TransformationProvider&,Classifier*&,BlobDetector*&,Camera::Type&);
  void init(TextLogger* tl){textlogger = tl;};

  uint16_t bestBallCandIndex;
  void findBall();
  float GetDistanceByBlobWidth(float);
  float GetDistanceByKinematics(int,int);
  float cameraZ();
  BallCandidate* candidates[MAX_BALL_CANDS];
  uint16_t ballCandCount;
  bool bestCandidateFound();
  void setHorizon(HorizonLine);
 private:
  bool checkAndSelectBall();
  bool sanityCheckBallCands(uint16_t*, WorldObject*);
  float checkPinkPct(int,int,int,int);
  bool checkSurroundingPink(int);
  float checkBelowGreenWhitePct(int);
  float checkColorsInCircleFit(int);
  void fitCircleToPoints(uint16_t*, uint16_t*, uint16_t, float*, float*, float*, float*);
  void fitCircleToBallCand(uint16_t, uint16_t, uint16_t*);
  void fitCircleToBallCands(uint16_t*, uint16_t*);
  void getBlobContour(uint16_t, uint16_t*, uint16_t*, uint16_t*);
  bool setBestBallCandAsBall(uint16_t*, WorldObject*);
  bool setBestBallCandidate(uint16_t*, WorldObject*, int);
  void sortOrangeBlobs(uint16_t*, uint16_t*);
  RobotVisionBlock*& robot_vision_;
  WorldObjectBlock*& world_objects_;
  BodyModelBlock*& body_model_;
  FrameInfoBlock*& vision_frame_info_;
  RobotStateBlock*& robot_state_;
  Vision::TransformationProvider& transform_;
  Classifier*& classifier_;
  BlobDetector*& blob_detector_;
  Blob **horizontalBlob, **verticalBlob;
  uint32_t *horizontalBlobCount, *verticalBlobCount;
  Camera::Type& camera_;
  TextLogger* textlogger;
  HorizonLine horizon_;
};

#endif
