#ifndef CAMERA_CALIBRATION_H
#define CAMERA_CALIBRATION_H

#include <common/RobotInfo.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <string>
#include <list>
#include <map>
#include <stdlib.h>
#include <constants/ImageConstants.h>

class CameraCalibration {
    public:
        Camera::Type type;
        float
            topCameraTiltOffset,
            topCameraRollOffset,
            topCameraYawOffset,
            bottomCameraTiltOffset,
            bottomCameraRollOffset,
            bottomCameraYawOffset,
            topCameraXOffset,
            topCameraZOffset,
            bottomCameraXOffset,
            bottomCameraZOffset,
            headTiltOffset,
            headPanOffset,
            topCameraFOVx,
            topCameraFOVy,
            bottomCameraFOVx,
            bottomCameraFOVy;

        CameraCalibration(){
            topCameraFOVx = bottomCameraFOVx = FOVx / DEG_T_RAD;
            topCameraFOVy = bottomCameraFOVy = FOVy / DEG_T_RAD;
        }

        CameraCalibration(const CameraCalibration& cc) {
            topCameraTiltOffset = cc.topCameraTiltOffset;
            topCameraRollOffset = cc.topCameraRollOffset;
            topCameraYawOffset = cc.topCameraYawOffset;
            bottomCameraTiltOffset = cc.bottomCameraTiltOffset;
            bottomCameraRollOffset = cc.bottomCameraRollOffset;
            bottomCameraYawOffset = cc.bottomCameraYawOffset;
            topCameraXOffset = cc.topCameraXOffset;
            topCameraZOffset = cc.topCameraZOffset;
            bottomCameraXOffset = cc.bottomCameraXOffset;
            bottomCameraZOffset = cc.bottomCameraZOffset;
            headTiltOffset = cc.headTiltOffset;
            headPanOffset = cc.headPanOffset;
            topCameraFOVx = cc.topCameraFOVx;
            topCameraFOVy = cc.topCameraFOVy;
            bottomCameraFOVx = cc.bottomCameraFOVx;
            bottomCameraFOVy = cc.bottomCameraFOVy;
        }
    bool loadFromFile(std::string file){
        std::ifstream inputfile;
        inputfile.open(file.c_str());
        if(!inputfile.good())
            return false;
        std::string key;
        double value = 0;
        while(readLine(inputfile, key, value)) {
            //std::cout << "key: " << key << ", value: " << value << "\n";
            if(key == "topCameraTiltOffset") topCameraTiltOffset = value;
            if(key == "topCameraRollOffset") topCameraRollOffset = value;
            if(key == "topCameraYawOffset") topCameraYawOffset = value;
            if(key == "bottomCameraTiltOffset") bottomCameraTiltOffset = value;
            if(key == "bottomCameraRollOffset") bottomCameraRollOffset = value;
            if(key == "bottomCameraYawOffset") bottomCameraYawOffset = value;
            if(key == "topCameraXOffset") topCameraXOffset = value;
            if(key == "topCameraZOffset") topCameraZOffset = value;
            if(key == "bottomCameraXOffset") bottomCameraXOffset = value;
            if(key == "bottomCameraZOffset") bottomCameraZOffset = value;
            if(key == "headTiltOffset") headTiltOffset = value;
            if(key == "headPanOffset") headPanOffset = value;
            if(key == "topCameraFOVx") topCameraFOVx = value;
            if(key == "topCameraFOVy") topCameraFOVy = value;
            if(key == "bottomCameraFOVx") bottomCameraFOVx = value;
            if(key == "bottomCameraFOVy") bottomCameraFOVy = value;
        }
        inputfile.close();
        return true;
    }

    void saveToFile(std::string file) {
        std::ofstream output;
        output.open(file.c_str());
        output << "topCameraTiltOffset:" << topCameraTiltOffset << "\n";
        output << "topCameraRollOffset:" << topCameraRollOffset << "\n";
        output << "topCameraYawOffset:" << topCameraYawOffset << "\n";
        output << "bottomCameraTiltOffset:" << bottomCameraTiltOffset << "\n";
        output << "bottomCameraRollOffset:" << bottomCameraRollOffset << "\n";
        output << "bottomCameraYawOffset:" << bottomCameraYawOffset << "\n";
        output << "topCameraXOffset:" << topCameraXOffset << "\n";
        output << "topCameraZOffset:" << topCameraZOffset << "\n";
        output << "bottomCameraXOffset:" << bottomCameraXOffset << "\n";
        output << "bottomCameraZOffset:" << bottomCameraZOffset << "\n";
        output << "headTiltOffset:" << headTiltOffset << "\n";
        output << "headPanOffset:" << headPanOffset << "\n";
        output << "topCameraFOVx:" << topCameraFOVx << "\n";
        output << "topCameraFOVy:" << topCameraFOVy << "\n";
        output << "bottomCameraFOVx:" << bottomCameraFOVx << "\n";
        output << "bottomCameraFOVy:" << bottomCameraFOVy << "\n";
    }
    private:
        bool readLine(std::ifstream& datafile, std::string& key, double& value) {
            std::string line;
            if(getline(datafile,line)) {
                int i = line.rfind(":");
                key = line.substr(0, i);
                std::string valueString = line.substr(i + 1, line.length() - i);
                value = atof(valueString.c_str());
                return true;
            }
            return false;
        }
};
#endif
