#ifndef CAMERA_CALIBRATOR_H
#define CAMERA_CALIBRATOR_H

#include <vision/CameraCalibration.h>
#include <vision/structures/Sample.h>
#include <math/Geometry.h>
#include <math/Pose2D.h>
#include <vector>
#include <vision/TransformationProvider.h>
#include <math/GaussNewtonOptimizer.h>
#include <memory/JointBlock.h>
#include <memory/WorldObjectBlock.h>
#include <memory/BodyModelBlock.h>

using namespace std;

class JointBlock;
class WorldObjectBlock;
class BodyModelBlock;

typedef Pose2D RobotPose;

class CameraCalibrator {
    private:
        Camera::Type type_;
        WorldObjectBlock* world_object_block_;
        BodyModelBlock* body_model_block_;
        JointBlock* joint_block_;
        vector<LineSegment> getFieldLinesCameraFrame(const RobotPose&, const Vision::TransformationProvider&) const;
        vector<LineSegment> getFieldLinesGlobalFrame() const;
        Vision::TransformationProvider top_transform_, bottom_transform_;
        void translateParameters(const vector<float>& parameters, CameraCalibration& cameraCalibration, RobotPose& robotPose) const;
        void translateParameters(const CameraCalibration& cameraCalibration, const RobotPose& robotPose, vector<float>& parameters) const;

        float getHeadPan() const;
        float getHeadTilt() const;
        float getFrontHeight() const;
        float rotation_, tilt_;
	public:
        CameraCalibrator(WorldObjectBlock* world_object_block, BodyModelBlock* body_model_block, JointBlock* joint_block) {
            world_object_block_ = world_object_block;
            body_model_block_ = body_model_block;
            joint_block_ = joint_block;
        }
        enum {
            topCameraTiltOffset,
            topCameraRollOffset,
            bottomCameraTiltOffset,
            bottomCameraRollOffset,
            //headTiltOffset,
            headPanOffset,
            poseX,
            poseY,
            //poseRotation,
            numOfParameterTranslations
        };
        float computeErrorParameterVector(const Sample& sample, const vector<float>& parameters);
        static vector<LineSegment> getFieldLinesCameraFrame(Pose3D, WorldObjectBlock*, const Vision::TransformationProvider&);

        void calibrate(CameraCalibration&,RobotPose&,vector<Sample>,int);
        float getSampleErrorCameraFrame(const Sample&, const RobotPose&) const;
        float getSampleErrorGlobalFrame(const Sample&, const RobotPose&) const;

        void setCameraZOffset(float);
        void setCameraXOffset(float);
};

#endif
