#ifndef CLASSIFIER_H
#define CLASSIFIER_H

#include <math/Point.h>
#include <list>
#include <vision/structures/VisionPoint.h>
#include <vision/structures/VisionParams.h>
#include <vision/enums/Colors.h>
#include <constants/ImageConstants.h>
#include <constants/VisionConstants.h>
#include <memory/ImageBlock.h>
#include <memory/RobotVisionBlock.h>
#include <memory/TextLogger.h>
#include <vision/ColorTableMethods.h>
#include <vision/structures/HorizonLine.h>

class Classifier {
 public:
  Classifier(ImageBlock*&, RobotVisionBlock*&,VisionParams&,Camera::Type&);

  void init(TextLogger* tl){textlogger = tl;};

  VisionPoint ***horizontalPoint, ***verticalPoint;
  uint32_t **horizontalPointCount, **verticalPointCount;

  void classifyImage(unsigned char*);
  void setHorizon(HorizonLine);
  void constructRuns(int colorFlags = ~0);
  void preProcessPoints();
  void preProcessGoalPoints();
  bool bodyExclusionAvailable;
  bool didHighResBallScan;


  /** High Res Scan Instructions:
    1. Call startHighResScan with the appropriate color. This will construct runs (i.e. VisionPoints).
    2. If it returns true then focus points were successfully identified and scanned.
    3. Run all blob construction and detection methods afterward.
    4. Run completeHighResScan to set the step sizes back to their default values. This will allow
       subsequent blob detections to proceed as if no high res scan had been taken.
    **/
  void setStepScale(int,int);
  void getStepSize(int&,int&);
  void getStepScale(int&,int&);
  bool startHighResScan(Color, int hStepScale = 0, int vStepScale = 0);
  bool startHighResGoalScan();
  bool startHighResBallScan();
  void completeHighResScan();
  void clearPreviousHighResScans();
  inline Color xy2color(int x, int y) {
    return (Color)segImg_[y * IMAGE_WIDTH + x];
  }
 private:
  bool prepareFocusMap(Color c);

  void completeHorizontalRun(uint8_t &hRunClr, int y, int hStartX, int finX = IMAGE_WIDTH);
  void completeVerticalRun(uint8_t &runClr, int x, int vStartY);

  void setImagePointers();
  void getFocusRanges(std::list<Point>&, int&, int&, int&, int&);

  ImageBlock*& image_;
  RobotVisionBlock*& robot_vision_;
  uint16_t vGreenPosition[IMAGE_WIDTH];
  uint16_t hGreenPosition[IMAGE_HEIGHT];
  VisionParams& params_;
  Camera::Type& camera_;
  TextLogger* textlogger;

  // For excluding parts of the shoulder in the body
  float bodyExclusionSlope[NUM_BODY_EXCL_POINTS];
  float bodyExclusionOffset[NUM_BODY_EXCL_POINTS];

  unsigned char* img_;
  unsigned char* segImg_;
  uint16_t vstep_, hstep_, vscale_, hscale_;
  HorizonLine horizon_;
  unsigned char* colorTable_;
  bool initialized_;
  bool pointScanned[IMAGE_SIZE];
  bool doingHighResScan_;
  bool focusMap_[IMAGE_SIZE];

  // Unused for now
  bool startFastHighResScan(Color, int hStepScale = 0, int vStepScale = 0);
  bool prepareFastFocusMap(Color c);
};
#endif
