#ifndef GOALDETECTOR_H
#define GOALDETECTOR_H

#include <memory/TextLogger.h>


#include <memory/WorldObjectBlock.h>
#include <memory/FrameInfoBlock.h>
#include <vision/structures/GoalPoint.h>
#include <vision/structures/FieldLine.h>
#include <vision/structures/CornerPoint.h>
#include <vision/enums/Colors.h>
#include <vision/BlobDetector.h>
#include <vision/LineDetector.h>
#include <vision/Classifier.h>

class GoalDetector {
 public:
  GoalDetector(WorldObjectBlock*&, FrameInfoBlock*&, Vision::TransformationProvider&, Classifier*&, BlobDetector*&,LineDetector*&, Camera::Type&);
  void init(TextLogger* tl){textlogger = tl;};

  float estimateGoalDistance(FieldLine * goal);
  void sanitizeGoalBlobs();
  float estimateGoalDistanceByKinematics(FieldLine* goal);
  float estimateGoalDistanceByHeight(float height);
  float estimateGoalDistanceByWidth(float width);
  void sanitizeGoalCandidate(FieldLine* goal);
  void setCornerPoints(CornerPoint**);
  void FormGoal();
  void FormYellowGoal();
  void resetYellowGoal();
  void setGoalObject(int goalIndex, float distance, float bearing, float elevation, int centerX, int centerY, float confidence, int lineIndex);
  void setCornerPointCounter(int value);
  bool mergeLinesVertical(FieldLine * lineTo, FieldLine * lineFrom);
  FieldLine** yellowPosts; // Goals
  int YellowPostCounter;
 private:
  WorldObjectBlock*& world_objects_;
  FrameInfoBlock*& vision_frame_info_;
  Vision::TransformationProvider& transform_;

  unsigned char color;

  CornerPoint** cornerPoints;

  Classifier*& classifier_;
  BlobDetector*& blob_detector_;
  LineDetector*& line_detector_;
  Camera::Type& camera_;
  TextLogger* textlogger;

  int totalValidGoals;
  FieldLine** currentLine;
  int cornerPointCounter;

  float greenWhitePercentBelow(FieldLine*);
};

#endif
