#ifndef IMAGEPROCESSOR_H
#define IMAGEPROCESSOR_H

#include <memory/TextLogger.h>
#include <vision/TransformationProvider.h>
#include <memory/JointBlock.h>
#include <memory/WorldObjectBlock.h>
#include <memory/BodyModelBlock.h>
#include <memory/ImageBlock.h>
#include <memory/RobotVisionBlock.h>
#include <memory/FrameInfoBlock.h>
#include <memory/RobotStateBlock.h>
#include <common/RobotInfo.h>
#include <vision/LineDetector.h>
#include <vision/GoalDetector.h>
#include <vision/BallDetector.h>
#include <vision/RobotDetector.h>
#include <vision/CrossDetector.h>
#include <vision/Classifier.h>
#include <vision/CameraCalibration.h>
#include <vision/structures/BallCandidate.h>
#include <math/Pose3D.h>

class ImageProcessor {
    public:
  ImageProcessor(WorldObjectBlock*&, BodyModelBlock*&, JointBlock*&, ImageBlock*&, RobotVisionBlock*&, FrameInfoBlock*&, RobotStateBlock*&, Camera::Type camera);
        void processFrame();
        void init(TextLogger*);
        void SetColorTable(unsigned char*);
        LineDetector* line_detector_;
        GoalDetector* goal_detector_;
        BallDetector* ball_detector_;
        BlobDetector* blob_detector_;
        RobotDetector* robot_detector_;
        CrossDetector* cross_detector_;
        Classifier* classifier_;
        unsigned char* getImg();
        unsigned char* getSegImg();
        unsigned char* getColorTable();
        bool isRawImageLoaded();
        int getImageHeight();
        int getImageWidth();
        const Vision::TransformationProvider& getTransform();
        void setCalibration(CameraCalibration);
        void updateTransform();
        std::vector<BallCandidate*> getBallCandidates();
        BallCandidate* getBestBallCandidate();
    private:
        float getTrueFrontHeight();
        int getTeamColor();
        double getCurrentTime();

        WorldObjectBlock*& world_objects_;
        BodyModelBlock*& body_model_;
        JointBlock*& joint_angles_;
        ImageBlock*& image_;
        FrameInfoBlock*& vision_frame_info_;
        RobotVisionBlock*& robot_vision_;
        RobotStateBlock*& robot_state_;

        Vision::TransformationProvider transform_;
        Camera::Type camera_;
        VisionParams visionParams;
        unsigned char* color_table_;
        TextLogger* textlogger;

        float getHeadPan() const;
        float getHeadTilt() const;
        float getHeadChange() const;

        CameraCalibration* calibration_;
};

#endif
