#ifndef ROBOTDETECTOR_H
#define ROBOTDETECTOR_H

#include <memory/TextLogger.h>
#include <memory/RobotVisionBlock.h>
#include <memory/WorldObjectBlock.h>
#include <memory/TextLogger.h>
#include <vision/TransformationProvider.h>
#include <vision/structures/Blob.h>
#include <vision/enums/Colors.h>
#include <vision/Macros.h>
#include <vision/BlobDetector.h>
#include <vision/Classifier.h>

#include <memory/RobotVisionBlock.h>

class RobotDetector {
 public:
  RobotDetector(WorldObjectBlock*&,RobotVisionBlock*&,Vision::TransformationProvider&,Classifier*&,BlobDetector*&,Camera::Type&);
  void init(TextLogger* tl){textlogger = tl;};
  void estimateBandBlobData();
  float estimateBandDistanceByWidth(Blob*);
  float estimateBandDistanceByGreenBelow(Blob*);
  float estimateBandDistance(Blob*);
  void sanitizeBandCandidate(Blob*,int);
  void sanitizeBandCandidateOnElevation(Blob*,float);
  void detectBlueRobots();
  void detectPinkRobots();
  void detectRobotCluster();
  std::list<Blob*> getBlueRobots();
  std::list<Blob*> getPinkRobots();
 private:
  Vision::TransformationProvider& transform_;
  Classifier*& classifier_;
  BlobDetector*& blob_detector_;
  RobotVisionBlock*& robot_vision_;
  WorldObjectBlock*& world_objects_;
  Camera::Type& camera_;
  TextLogger* textlogger;
  std::list<Blob*> blueRobots_, pinkRobots_;
};
#endif
