#ifndef VISION_99KDYIX5
#define VISION_99KDYIX5

#include <Module.h>
#include <common/RobotInfo.h>

#include <common/WorldObject.h>
#include <common/ColorSpaces.h>

#include <vision/TransformationProvider.h>
#include <vision/LineDetector.h>
#include <vision/GoalDetector.h>
#include <vision/BlobDetector.h>
#include <vision/Classifier.h>
#include <vision/ImageProcessor.h>

#include <vision/structures/FieldLine.h>
#include <vision/structures/LinePoint.h>
#include <vision/structures/CornerPoint.h>
#include <vision/structures/LineStackInfo.h>
#include <vision/structures/LineStackRet.h>
#include <vision/structures/VisionParams.h>
#include <vision/structures/Blob.h>
#include <vision/structures/VisionPoint.h>
#include <vision/enums/Colors.h>
#include <vision/enums/VisionBodyPoints.h>
#include <vision/Macros.h>
#include <constants/VisionConstants.h>

class FrameInfoBlock;
class JointBlock;
class RobotVisionBlock;
class SensorBlock;
class ImageBlock;
class WorldObjectBlock;
class RobotStateBlock;
class BodyModelBlock;
class CameraBlock;
class RobotInfoBlock;

class VisionModule: public Module {

public:
  VisionModule();
  ~VisionModule();

  void specifyMemoryBlocks();
  void specifyMemoryDependency();
  void initSpecificModule();
  void processFrame();
  void updateTransforms();

  bool loadColorTables();
  bool loadColorTable(Camera::Type camera, std::string fileName, bool fullpath=false);
  void loadCalibration();

  ImageProcessor *top_processor_, *bottom_processor_;

  unsigned char* bottomColorTable;
  unsigned char* topColorTable;
  string bottomColorTableName;
  string topColorTableName;

private:
  FrameInfoBlock *vision_frame_info_;
  JointBlock *joint_angles_;
  RobotVisionBlock *robot_vision_;
  SensorBlock *sensors_;
  ImageBlock *image_;
  WorldObjectBlock *world_objects_;
  RobotStateBlock *robot_state_;
  BodyModelBlock *body_model_;
  CameraBlock *camera_info_;
  RobotInfoBlock *robot_info_;

  bool isBottomCamera();
  bool useSimColorTable();
  std::string getDataBase();
  int getRobotId();
  int getTeamColor();
};

#endif /* end of include guard: VISION_99KDYIX5 */
