#ifndef HORIZON_LINE_H
#define HORIZON_LINE_H

#include <vision/TransformationProvider.h>

struct HorizonLine {
  HorizonLine() { exists = false; }
  bool exists;
  float gradient;
  float offset;
  static HorizonLine generate(Vision::TransformationProvider& transform, int distance, float trueFrontHeight) {

    // The horizon is the line drawn on the ground directly in front of the camera at a distance of 8 meters

    float d, bearing, e;
    transform.transformPolar(5000, 0, 0, d, bearing, e);

    float bearingLeft = bearing + FOVx / 2;
    float bearingRight = bearing - FOVx / 2;

    float z = -trueFrontHeight;
    float lx = distance * cosf(bearingLeft);
    float ly = distance * sinf(bearingLeft);
    float rx = distance * cosf(bearingRight);
    float ry = distance * sinf(bearingRight);

    float ldist, lbear, lelev;
    float rdist, rbear, relev;
    transform.inverseTransform(lx, ly, z, ldist, lbear, lelev);
    transform.inverseTransform(rx, ry, z, rdist, rbear, relev);

    float x1, x2, y1, y2;

    x1 = transform.invertBearing(lbear);
    y1 = transform.invertElevation(lelev);

    x2 = transform.invertBearing(rbear);
    y2 = transform.invertElevation(relev);

    HorizonLine horizon;
    horizon.gradient = (y2 - y1) / (x2 - x1);
    if (x2 - x1 != 0) {
      horizon.offset = y1 - horizon.gradient * x1;
    } else {
      horizon.offset = 0;
    }
    horizon.exists = true;

    return horizon;
  }
  bool isAbovePoint(int imageX, int imageY) {
    float yCoordinate = imageX * gradient + offset;
    return imageY > yCoordinate; // y axis is flipped
  }
};

#endif
