/**
 * @author Michael Quinlan
 *
 * Version : $Id$
 * This file was generated by Aldebaran Robotics ModuleGenerator
 */

#include "naointerface.h"

#include <alvalue/alvalue.h>
#include <alcommon/alproxy.h>
#include <alcommon/albroker.h>

#include <almemoryfastaccess/almemoryfastaccess.h>
#include <alproxies/dcmproxy.h>
#include <alproxies/almemoryproxy.h>

#include <alproxies/altexttospeechproxy.h>

#include <boost/thread.hpp>
#include <boost/lexical_cast.hpp>

#include <common/Poses.h>
#include <common/RobotInfo.h>
#include <common/InterfaceInfo.h>

/**
 * Constructor for naointerface object
 * @param broker The parent broker
 * @param name The name of the module
 */
naointerface::naointerface( AL::ALPtr<AL::ALBroker> broker, const std::string& name):
  AL::ALModule(broker, name)
{
  std::cout << "NaoInterface::Starting Interface Module" << std::endl << std::flush;

  fast_sensor_access_ = (AL::ALPtr<AL::ALMemoryFastAccess>(new AL::ALMemoryFastAccess()));
  dcmWrap_ = new DCMWrapper();

  initMemory();
  // set up locks
  cleanLock(Lock::getLockName(memory_,LOCK_MOTION));
  motion_lock_ = new Lock(Lock::getLockName(memory_,LOCK_MOTION));

  setModuleDescription("Interface between the robot and the Austin Villa Code base");
  start();
}

/**
 * Destructor for naointerface object
 */
naointerface::~naointerface() {}

void naointerface::start() {
  // Create the proxy to dcm
  try {
    std::cout << "NaoInterface::Creating DCM Proxy\n" << std::flush;
    // Get the DCM proxy
    //dcm_proxy_ = getParentBroker()->getDcmProxy();
    dcmWrap_->setProxy(getParentBroker()->getDcmProxy());
  } catch (AL::ALError& e) {
    throw ALERROR(getName(), "start()", "Impossible to create DCM Proxy : " + e.toString());
  }

  try {
    std::cout << "NaoInterface::Creating Text to Speech Proxy\n" << std::flush;
    tts_proxy_ = AL::ALPtr<AL::ALTextToSpeechProxy>(new AL::ALTextToSpeechProxy(getParentBroker()));
    tts_proxy_->setVolume(1.0);
    std::cout << "DONE" << std::flush << std::endl;
  } catch (AL::ALError& e) {
    throw ALERROR(getName(), "start()", "Impossible to create TextToSpeech Proxy : " + e.toString());
  }

  initFastAccess();
  dcmWrap_->init();
  
  // create sonar
  initSonar();

  // Connect callback to the DCM post proccess
  try {
    std::cout << "NaoInterface::Creating DCM Callback\n" << std::flush;
    dcm_postprocess_connection_ = getParentBroker()->getProxy("DCM")->getModule()->atPostProcess(boost::bind(&naointerface::postProcess, this));
    dcm_preprocess_connection_ = getParentBroker()->getProxy("DCM")->getModule()->atPreProcess(boost::bind(&naointerface::preProcess, this));
  } catch (const AL::ALError &e) {
    throw ALERROR(getName(), "run()", "Error when connecting to DCM postProccess: " + e.toString());
  }

  tts_proxy_->post.say("Interface");
}

void naointerface::stop() {
}

double naointerface::getSystemTime() {
  struct timezone tz;
  timeval timeT;
  gettimeofday(&timeT, &tz);
  return timeT.tv_sec + (timeT.tv_usec / 1000000.0);
}

// will be called every 10ms : main cycle
void naointerface::postProcess() {
  // ####### !!!WARNING!!! WE ARE IN REAL-TIME HERE #######
  bool res;
  res = motion_lock_->timed_lock(9);
  if (!res)
    std::cout << "WARNING: Didn't acquire lock in postProcess, but continuing anyway" << std::endl << std::flush;
  // set the frame time and id
  frame_info_->seconds_since_start = getSystemTime() - frame_info_->start_time;
  frame_info_->frame_id++;
  
  populateSensors();

  if (res)
    motion_lock_->unlock();
  motion_lock_->notify_one();

  if (speech_->say_text_) {
    speech_->say_text_ = false;
    if ((frame_info_->frame_id - speech_->last_speech_frame_) > 200){
      speech_->last_speech_frame_ = frame_info_->frame_id;
      tts_proxy_->stopAll();
      tts_proxy_->post.say(speech_->text_);
    }
  }
}

void naointerface::preProcess() {
  // ####### !!!WARNING!!! WE ARE IN REAL-TIME HERE #######
  // Execute the commands
  bool res;
  res = motion_lock_->timed_lock(9);
  if (!res)
    std::cout << "WARNING: Didn't acquire lock in preProcess, but continuing anyway" << std::endl << std::flush;
  
  dcmWrap_->sendToActuators(raw_joint_commands_,raw_joint_angles_);
  dcmWrap_->sendToLEDs(led_commands_);

  if (res)
    motion_lock_->unlock();
  motion_lock_->notify_one();
}

void naointerface::initMemory() {
  // setup the memory
  memory_ = new Memory(true,MemoryOwner::INTERFACE,0,1,true);
  
  memory_->getOrAddBlockByName(frame_info_,"frame_info");
  memory_->getOrAddBlockByName(raw_sensors_,"raw_sensors");
  memory_->getOrAddBlockByName(raw_joint_angles_,"raw_joint_angles");
  memory_->getOrAddBlockByName(processed_joint_angles_,"processed_joint_angles");
  memory_->getOrAddBlockByName(raw_joint_commands_,"raw_joint_commands");
  memory_->getOrAddBlockByName(processed_joint_commands_,"processed_joint_commands");
  memory_->getOrAddBlockByName(led_commands_,"led_commands",MemoryOwner::VISION);
  memory_->getOrAddBlockByName(speech_,"speech",MemoryOwner::SHARED);

  // turn on center button to white so we know naoqi is up
  led_commands_->values_[ChestRed]=1.0;
  led_commands_->values_[ChestGreen]=1.0;
  led_commands_->values_[ChestBlue]=1.0;
  
  // set the start time
  frame_info_->source = MEMORY_ROBOT;
  frame_info_->start_time = getSystemTime();

  // Initialise robot to sitting pose
  //processed_joint_commands_->setPoseDeg(Pose::sitting);
  for (int i = 0; i < NUM_JOINTS; i++)
    processed_joint_commands_->angles_[i] = processed_joint_angles_->values_[i];
  for (int i = 0; i < NUM_JOINTS; i++)
    processed_joint_commands_->stiffness_[i] = -1;
  processed_joint_commands_->body_angle_time_ = 1000;
  processed_joint_commands_->head_pitch_angle_time_ = 1000;
  processed_joint_commands_->head_yaw_angle_time_ = 1000;
  processed_joint_commands_->stiffness_time_ = 1000;
  processed_joint_commands_->send_stiffness_ = false;
  processed_joint_commands_->send_head_pitch_angle_ = false;
  processed_joint_commands_->send_head_yaw_angle_ = false;

}

// ALMemory fast access
void naointerface::initFastAccess() {
  std::cout << "NaoInterface::Initiating Fast Access to Sensors\n" << std::flush;
  sensor_keys_.clear();
  
  // Joints Position list
  for (int i = 0; i < NUM_JOINTS; i++) {
    sensor_keys_.push_back(std::string("Device/SubDeviceList/") + getJointName((Joint)i) + "/Position/Sensor/Value");
  }

  // Sensor Values list
  for (int i = 0; i < NUM_SENSORS; i++) {
    sensor_keys_.push_back(std::string("Device/SubDeviceList/") + getSensorString((Sensor)i) + "/Sensor/Value");
  }
  
  // Sonar Values list
  std::string num;
  for (int i = 0; i < NUM_SONAR_VALS; i++) {
    if (i == 0)
      num = "";
    else
      num = boost::lexical_cast<std::string>(i);
    sensor_keys_.push_back(std::string("Device/SubDeviceList/US/Left/Sensor/Value") + num);
  }
  for (int i = 0; i < NUM_SONAR_VALS; i++) {
    if (i == 0)
      num = "";
    else
      num = boost::lexical_cast<std::string>(i);
    sensor_keys_.push_back(std::string("Device/SubDeviceList/US/Right/Sensor/Value") + num);
  }

  // Joints temperature list
  for (int i = 0; i < NUM_JOINTS; i++) {
    sensor_keys_.push_back(std::string("Device/SubDeviceList/") + getJointName((Joint)i) + "/Temperature/Sensor/Value");
  }

  // Joints stiffness list
  for (int i = 0; i < NUM_JOINTS; i++) {
    sensor_keys_.push_back(std::string("Device/SubDeviceList/") + getJointName((Joint)i) + "/Hardness/Actuator/Value");
  }

  // Create the fast memory access
  fast_sensor_access_->ConnectToVariables(getParentBroker(), sensor_keys_, false);
}

void naointerface::populateSensors(){
  //Get data from the fast sensor proxy
  fast_sensor_access_->GetValues(sensor_values_);

  int offset = 0;

  for (int i = 0; i < NUM_JOINTS; i++)
    raw_joint_angles_->values_[i] = sensor_values_[i + offset];
  raw_joint_angles_->values_[RHipYawPitch] = raw_joint_angles_->values_[LHipYawPitch];
  offset += NUM_JOINTS;

  for (int i = 0; i < NUM_SENSORS; i++)
    raw_sensors_->values_[i] = sensor_values_[i + offset];
  offset += NUM_SENSORS;
  
  for (int i = 0; i < NUM_SONAR_VALS; i++)
    raw_sensors_->sonar_left_[i] = sensor_values_[i + offset];
  offset += NUM_SONAR_VALS;
  for (int i = 0; i < NUM_SONAR_VALS; i++)
    raw_sensors_->sonar_right_[i] = sensor_values_[i + offset];
  offset += NUM_SONAR_VALS;
  
  for (int i = 0; i < NUM_JOINTS; i++)
    raw_sensors_->joint_temperatures_[i] = sensor_values_[i + offset];
  raw_sensors_->joint_temperatures_[RHipYawPitch] = raw_sensors_->joint_temperatures_[LHipYawPitch];
  offset += NUM_JOINTS;

  for (int i = 0; i < NUM_JOINTS; i++)
    raw_joint_angles_->stiffness_[i] = sensor_values_[i + offset];
  raw_joint_angles_->stiffness_[RHipYawPitch] = raw_joint_angles_->stiffness_[LHipYawPitch];
  offset += NUM_JOINTS;

  //raw_sensors_->values_[accelX] *= -9.81 / 56.0; // because aldebaran has them zeroed at 56
  //raw_sensors_->values_[accelY] *= -9.81 / 56.0; // because aldebaran has them zeroed at 56
  //raw_sensors_->values_[accelZ] *= -9.81 / 56.0; // because aldebaran has them zeroed at 56

  //std::cout << "SENSORS: ";
  //for (int i = 0; i < NUM_SENSORS; i++) 
    //std::cout << raw_sensors_->values_[i] << " ";
  //std::cout << std::endl;
}


void naointerface::initSonar(){
  std::cout << "Initializing UltraSound ... " << std::flush;

  try {
    dcmWrap_->initSonar();
  } catch( AL::ALError& e ) {
    std::cout << "FAILED! - " << e.what() << std::endl;
    return;
  }
  std::cout << "Done !" << std::endl;

} 
