#ifndef IMAGE_WIDGET_H
#define IMAGE_WIDGET_H

#include <QtGui>
#include <QWidget>
#include <QImage>

#include <vector>
#include <math.h>
#include <iostream>

#include <common/RobotInfo.h>

#include "Annotations/SelectionType.h"
#include "Annotations/SelectionMode.h"
#include "Annotations/EllipseSelection.h"
#include "Annotations/PolygonSelection.h"
#include "Annotations/RectangleSelection.h"
#include "Annotations/Selection.h"

#define UTPolygon PolygonSelection
#define UTEllipse EllipseSelection
#define UTRectangle RectangleSelection

class QPaintEvent;

class ImageWidget : public QWidget {
  Q_OBJECT
  Q_PROPERTY(bool selectionEnabled READ getSelectionEnabled WRITE setSelectionEnabled)

  public:
   ImageWidget(QWidget *parent);
   void setImageSize(int,int);

   void paintEvent(QPaintEvent *event);
   void mousePressEvent(QMouseEvent *event);
   void mouseReleaseEvent(QMouseEvent *event);
   void mouseMoveEvent(QMouseEvent *event);

   void setSelectionType(SelectionType);
   bool getSelectionEnabled();

  QImage* img;

  public slots:
    void selectionTypeChanged(SelectionType);
    void setSelectionEnabled(bool);
    void setCurrentSelections(std::vector<Selection*>);

  signals:
    void clicked(int x, int y, int button);
    void mouseXY(int x, int y);
    void selected(Selection*);
  private:
    int width_, height_;
    bool isDragging_, isSelecting_, isSelectionEnabled_;
    SelectionType selectionType_;
    SelectionMode selectionMode_;

    void updateSelection(int,int);
    void beginSelection();
    void endSelection();
    void clearSelection();
    void drawSelection();
    void drawSelectionRectangle(bool);
    void drawSelectionPolygon();
    void drawSelectionEllipse();
    void drawStoredSelections();
    void drawPolygon(UTPolygon*);
    void drawEllipse(UTEllipse*);
    void drawRectangle(UTRectangle*,bool);

    void zoomIn(int,int,int,int);
    void zoomOut();
    QRect getViewBox();
    QPoint mapPoint(QPoint);
    QPoint unMapPoint(QPoint);

    SelectionMode getMode();
    SelectionType getType();
    int startX_, startY_, currentX_, currentY_;
    std::vector<QRect> zoomstack_;
    std::vector<QPoint> selectionVertices_;
    std::vector<Selection*> selections_;
    QColor selectionColor_;
};

#endif
