#include <QApplication>
#include <iostream>

#include "UTMainWnd.h" 
#include "FilesWindow.h" 

namespace {

  bool displayMotionWindow = false;
  bool displayVisionWindow = false;
  bool displayFilesWindow = false;
  bool displayWorldWindow = false;
  bool displayPlotWindow = false;
  bool displayTeamWindow = false;
  bool openRecent = false;
  std::string logFile;
  QString IP;

  int startFrame = 0;
  int endFrame = -1;
}

void displayHelp() {
  std::cout << "\n\nOptions :\n";
  std::cout << "\t-o <file>\topens log file\n";
  std::cout << "\t-m\topens motion window\n";
  std::cout << "\t-v\topens vision window\n";
  std::cout << "\t-f\topens files window\n";
  //std::cout << "\t-c\topens control window\n";
  std::cout << "\t-w\topens world window\n";
  //std::cout << "\t-l\topens log window\n";
  //std::cout << "\t-v\topens vision window\n";
  //std::cout << "\t-k\topens kinematics window\n";
  //std::cout << "\t-z\tdefaults to run core\n";
  std::cout << "\t-r\topens most recent log\n";
  std::cout << "\t-t\topens team cfg window\n";
  std::cout << "\t-s\tselect start frame\n";
  std::cout << "\t-e\tselect end frame\n";
  std::cout << "\t-i <0-9>\tinitializes with IP set to 192.168.25.9x\n";
  std::cout << "\t-c <0-9>\tinitializes with IP set to 11.0.1.9x\n";
  //std::cout << "\t-n\tdo NOT run core (for slow machines)\n";
  std::cout << "\t-h -?\t display this help\n\n";
}

int getParameters(int argc, char **argv) {
  char ch;
  const char* optflags = "mpvfwrto:i:c:s:e:h?";
  while(-1 != (ch = getopt(argc, argv, optflags))) {
    switch(ch) {
    case 'm':
      displayMotionWindow = true;
      break;
    case 'p':
      displayPlotWindow = true;
      break;
    case 'v':
      displayVisionWindow = true;
      break;
    case 'f':
      displayFilesWindow = true;
      break;
    case 'w':
      displayWorldWindow = true;
      break;
    case 't':
      displayTeamWindow = true;
      break;
    case 'o':
      logFile = std::string(optarg);
      break;
    case 'r':
      openRecent = true;
      break;
    case 'i':
      IP = "192.168.25.9"+QString(optarg);
      break;
    case 'c':
      IP = "11.0.1.9"+QString(optarg);
      break;
    case 's':
      startFrame = atoi(optarg);
      break;
    case 'e':
      endFrame = atoi(optarg);
      break;
      
    case 'h':
    case '?':
      displayHelp();
      return 0;
    }
  }
  return 1;
}

int main(int argc, char **argv) {

  if (!getParameters(argc, argv))
    return 0;

  QApplication a(argc, argv);

  UTMainWnd* main;
  if (logFile.empty()) {
    main = new UTMainWnd(NULL);
  } else {
    main = new UTMainWnd(logFile.c_str());
  }

  main->setFrameRange(startFrame, endFrame);

  if (IP.size() > 0){
    std::cout << "IP : " << IP.toStdString() << std::endl;
    main->filesWnd_->setCurrentLocation(IP);
  }

  if (openRecent) main->openRecent();

  main->show();
  
  if (displayMotionWindow)
    main->openMotionWnd();
  if (displayPlotWindow)
    main->openPlotWnd();
  if (displayVisionWindow)
    main->openVisionWnd();
  if (displayFilesWindow)
    main->openFilesWnd(); 
  if (displayWorldWindow)
    main->openWorldWnd();
  if (displayTeamWindow)
    main->openTeamWnd();

  a.exec();

  delete main;
  return 0;
}
