float VisionWindow::getTopTiltOffset() const{
    return calibrationWnd_->topTiltOffset->value();
}

void VisionWindow::setTopTiltOffset(float value) {
    calibrationWnd_->topTiltOffset->setValue(value);
}

float VisionWindow::getTopRollOffset() const{
    return calibrationWnd_->topRollOffset->value();
}

void VisionWindow::setTopRollOffset(float value) {
    calibrationWnd_->topRollOffset->setValue(value);
}

float VisionWindow::getTopXOffset() const{
    return calibrationWnd_->topXOffset->value();
}

void VisionWindow::setTopXOffset(float value) {
    calibrationWnd_->topXOffset->setValue(value);
}

float VisionWindow::getTopZOffset() const{
    return calibrationWnd_->topZOffset->value();
}

void VisionWindow::setTopZOffset(float value) {
    calibrationWnd_->topZOffset->setValue(value);
}

float VisionWindow::getBottomTiltOffset() const{
    return calibrationWnd_->bottomTiltOffset->value();
}

void VisionWindow::setBottomTiltOffset(float value) {
    calibrationWnd_->bottomTiltOffset->setValue(value);
}

float VisionWindow::getBottomRollOffset() const{
    return calibrationWnd_->bottomRollOffset->value();
}

void VisionWindow::setBottomRollOffset(float value) {
    calibrationWnd_->bottomRollOffset->setValue(value);
}

float VisionWindow::getBottomXOffset() const{
    return calibrationWnd_->bottomXOffset->value();
}

void VisionWindow::setBottomXOffset(float value) {
    calibrationWnd_->bottomXOffset->setValue(value);
}

float VisionWindow::getBottomZOffset() const{
    return calibrationWnd_->bottomZOffset->value();
}

void VisionWindow::setBottomZOffset(float value) {
    calibrationWnd_->bottomZOffset->setValue(value);
}

void VisionWindow::setPoseX(float value) {
    calibrationWnd_->poseX->setValue(value);
}

float VisionWindow::getPoseX() const{
    return calibrationWnd_->poseX->value();
}

void VisionWindow::setPoseY(float value) {
    calibrationWnd_->poseY->setValue(value);
}

float VisionWindow::getPoseY() const{
    return calibrationWnd_->poseY->value();
}

void VisionWindow::setPoseRotation(float value) {
    calibrationWnd_->poseRotation->setValue(value);
}

float VisionWindow::getPoseRotation() const{
    return calibrationWnd_->poseRotation->value();
}

void VisionWindow::setHeadPanOffset(float value) {
    calibrationWnd_->headPanOffset->setValue(value);
}

float VisionWindow::getHeadPanOffset() const{
    return calibrationWnd_->headPanOffset->value();
}

void VisionWindow::setHeadTiltOffset(float value) {
    calibrationWnd_->headTiltOffset->setValue(value);
}

float VisionWindow::getHeadTiltOffset() const{
    return calibrationWnd_->headTiltOffset->value();
}


void VisionWindow::getTilts(float &topCamTilt, float &botCamTilt) {
  topCamTilt = getTopTiltOffset();
  botCamTilt = getBottomTiltOffset();
}

float VisionWindow::getBodyRoll() {
  if (sensor_block_ == NULL)
    return 0;
  return sensor_block_->values_[angleX];
}

float VisionWindow::getBodyTilt() {
  if (sensor_block_ == NULL)
    return 0;
  return sensor_block_->values_[angleY];
}

float VisionWindow::getHeadTilt() {
  if (joint_block_ == NULL)
    return 0;
  return -joint_block_->values_[HeadTilt];
}

float VisionWindow::getHeadPan() {
  if (joint_block_ == NULL)
    return 0;
  return joint_block_->values_[HeadPan];
}

float VisionWindow::getTrueFrontHeight() const {
  if (body_model_block_ == NULL)
    return 0;
  return body_model_block_->abs_parts_[BodyPart::torso].translation.z;
}

Pose3D VisionWindow::getHeadMatrix() {
  if (body_model_block_ == NULL)
    return Pose3D(0,0,0);
  return body_model_block_->rel_parts_[BodyPart::head];
}

Pose3D VisionWindow::getTorsoMatrix() {
  if (body_model_block_ == NULL)
    return Pose3D(0,0,0);
  return body_model_block_->rel_parts_[BodyPart::torso];
}

float VisionWindow::getBallDistance(float groundDistance) {
  float ballRadius = 31;
  float camZ = body_model_block_->rel_parts_[BodyPart::top_camera].translation.z - ballRadius;
  return sqrt(camZ * camZ + groundDistance * groundDistance);
}

vector<LineSegment> VisionWindow::getCalibrationLineSegments(ImageWidget* image) {
  ImageProcessor* processor = getImageProcessor(image);
  vector<LineSegment> segments;
  Pose3D position(getPoseX(), getPoseY(), 0);
  position.rotateZ(getPoseRotation() * DEG_T_RAD);
  return CameraCalibrator::getFieldLinesCameraFrame(position, world_object_block_, processor->getTransform());
}

Point2D VisionWindow::getNearestLinePoint(ImageWidget* image, Sample s) {
    vector<LineSegment> segments = getCalibrationLineSegments(image);
    if(segments.size() == 0){
        return s;
    }
    Point2D nearest = segments[0].getNearestPoint(s);
    float minDistance = (s - nearest).getMagnitude();
    for(unsigned int i=0; i<segments.size(); i++){
        LineSegment segment = segments[i];
        Point2D p = segment.getNearestPoint(s);
        float distance = (s - p).getMagnitude();
        if(minDistance > distance){
            minDistance = distance;
            nearest = p;
        }
    }
    return nearest;
}

RobotPose VisionWindow::getPose() const {
    RobotPose pose;
    pose.translation.x = getPoseX();
    pose.translation.y = getPoseY();
    pose.rotation = getPoseRotation();
    return pose;
}

void VisionWindow::setPose(const RobotPose& pose) {
    setPoseX(pose.translation.x);
    setPoseY(pose.translation.y);
    setPoseRotation(pose.rotation);
}

CameraCalibration VisionWindow::getCalibration() const {
    return calibrationWnd_->getCalibration();
}

void VisionWindow::setCalibration(const CameraCalibration& cal) {
    calibrationWnd_->loadCalibration(cal);
}

